@@ -16,15 +16,15 @@
 
 #include "PlaylistBrowser.h"
 
+#include "browsers/playlistbrowser/APGCategory.h"
+#include "browsers/playlistbrowser/DynamicCategory.h"
+#include "browsers/playlistbrowser/UserPlaylistCategory.h"
+#include "core/playlists/Playlist.h"
 #include "core/support/Amarok.h"
 #include "core/support/Debug.h"
-#include "APGCategory.h"
-#include "DynamicCategory.h"
-#include "core/playlists/Playlist.h"
-#include "UserPlaylistCategory.h"
-#include "PlaylistManager.h"
+#include "playlistmanager/PlaylistManager.h"
 
-#include <klocale.h>
+#include <KLocale>
 #include <KStandardDirs>
 
 #include <QList>
@@ -36,11 +36,9 @@ PlaylistBrowserNS::PlaylistBrowser::PlaylistBrowser( const QString &name, QWidge
     setMargin( 0 );
     setContentsMargins(0,0,0,0);
 
-    BrowserCategoryList::addCategory( new DynamicCategory( 0 ) );
-    BrowserCategoryList::addCategory( new UserPlaylistCategory( 0 ) );
-    BrowserCategoryList::addCategory( new APGCategory( 0 ) );
-
-    connect( The::playlistManager(), SIGNAL( categoryAdded( int ) ), SLOT( addCategory( int ) ) );
+    addCategory( new DynamicCategory( 0 ) );
+    addCategory( new UserPlaylistCategory( 0 ) );
+    addCategory( new APGCategory( 0 ) );
 
     setLongDescription( i18n( "The playlist browser contains your list of imported and saved playlists. It is also where you can specify powerful dynamic playlists and manage your podcast subscriptions and episodes." ) );
 
@@ -51,11 +49,4 @@ PlaylistBrowserNS::PlaylistBrowser::~PlaylistBrowser()
 {
 }
 
-//SLOT
-void
-PlaylistBrowserNS::PlaylistBrowser::addCategory( int )
-{
-    DEBUG_BLOCK
-}
-
 #include "PlaylistBrowser.moc"
