@@ -30,19 +30,14 @@
 #include "ScriptManager.h"
 
 #include <KConfigDialog>
-#include <KTextBrowser>
 #include <KStandardDirs>
+#include <KTextBrowser>
 
 #include <Plasma/IconWidget>
-#include <Plasma/TextBrowser>
-#include <Plasma/TreeView>
 
 #include <QAction>
 #include <QGraphicsLinearLayout>
-#include <QPainter>
-#include <QPoint>
 #include <QScrollBar>
-#include <QStandardItemModel>
 #include <QTimer>
 
 class LyricsAppletPrivate
@@ -143,7 +138,6 @@ LyricsAppletPrivate::determineActionIconsState()
 void
 LyricsAppletPrivate::showLyrics( const QString &text )
 {
-    DEBUG_BLOCK
     browser->clear();
     browser->setLyrics( text );
     showSuggestions = false;
@@ -209,7 +203,7 @@ LyricsAppletPrivate::showUnsavedChangesWarning( Meta::TrackPtr newTrack )
     }
 
     // Show the warning message.
-    q->showWarning( warningMessage, SLOT(_lyricsChangedMessageButtonPressed(const Plasma::MessageButton)) );
+    q->showWarning( warningMessage, SLOT(_lyricsChangedMessageButtonPressed(Plasma::MessageButton)) );
 
     // Make the contents readonly again.
     // Since the applet is now blocked the user can not enable this again.
@@ -377,8 +371,6 @@ LyricsAppletPrivate::_unsetCursor()
 void
 LyricsAppletPrivate::_trackDataChanged( Meta::TrackPtr track )
 {
-    DEBUG_BLOCK
-
     userAutoScrollOffset = 0;
     oldSliderPosition = 0;
 
@@ -479,7 +471,7 @@ LyricsApplet::init()
     autoScrollAction->setEnabled( true );
     autoScrollAction->setText( i18n( "Scroll automatically" ) );
     d->autoScrollIcon = addRightHeaderAction( autoScrollAction );
-    connect( d->autoScrollIcon, SIGNAL( clicked() ), this, SLOT( _toggleAutoScroll() ) );
+    connect( d->autoScrollIcon, SIGNAL(clicked()), this, SLOT(_toggleAutoScroll()) );
 
     QAction* reloadAction = new QAction( this );
     reloadAction->setIcon( KIcon( "view-refresh" ) );
@@ -520,9 +512,9 @@ LyricsApplet::init()
 
     EngineController* engine = The::engineController();
 
-    connect( engine, SIGNAL( trackChanged( Meta::TrackPtr ) ), this, SLOT( _trackDataChanged( Meta::TrackPtr ) ) );
-    connect( engine, SIGNAL( trackMetadataChanged( Meta::TrackPtr ) ), this, SLOT( _trackDataChanged( Meta::TrackPtr ) ) );
-    connect( engine, SIGNAL( trackPositionChanged( qint64 , bool) ), this, SLOT( _trackPositionChanged(qint64, bool) ) );
+    connect( engine, SIGNAL(trackChanged(Meta::TrackPtr)), this, SLOT(_trackDataChanged(Meta::TrackPtr)) );
+    connect( engine, SIGNAL(trackMetadataChanged(Meta::TrackPtr)), this, SLOT(_trackDataChanged(Meta::TrackPtr)) );
+    connect( engine, SIGNAL(trackPositionChanged(qint64,bool)), this, SLOT(_trackPositionChanged(qint64,bool)) );
     connect( d->suggestView, SIGNAL(selected(LyricsSuggestion)), SLOT(_suggestionChosen(LyricsSuggestion)) );
     connect( dataEngine("amarok-lyrics"), SIGNAL(sourceAdded(QString)), this, SLOT(connectSource(QString)) );
 
@@ -553,7 +545,6 @@ LyricsApplet::connectSource( const QString& source )
 void
 LyricsApplet::dataUpdated( const QString& name, const Plasma::DataEngine::Data& data )
 {
-    DEBUG_BLOCK
     Q_D( LyricsApplet );
 
     if( name != QLatin1String("lyrics") )
@@ -663,7 +654,7 @@ LyricsApplet::refreshLyrics()
     {
         // Ask the user if he really wants to refetch the lyrics.
         const QString text( i18nc( "@info", "Do you really want to refetch lyrics for this track? All changes you may have made will be lost.") );
-        showWarning( text, SLOT(_refetchMessageButtonPressed(const Plasma::MessageButton)) );
+        showWarning( text, SLOT(_refetchMessageButtonPressed(Plasma::MessageButton)) );
     }
     else
     {
@@ -677,6 +668,9 @@ void
 LyricsApplet::createConfigurationInterface( KConfigDialog *parent )
 {
     Q_D( LyricsApplet );
+
+    parent->setButtons( KDialog::Ok | KDialog::Cancel );
+
     KConfigGroup configuration = config();
     QWidget *settings = new QWidget;
     d->ui_settings.setupUi( settings );
@@ -698,7 +692,6 @@ LyricsApplet::createConfigurationInterface( KConfigDialog *parent )
         break;
     }
 
-    parent->enableButtonApply( true );
     parent->addPage( settings, i18n( "Lyrics Settings" ), "preferences-system" );
 
     connect( parent, SIGNAL(accepted()), this, SLOT(_changeLyricsFont()) );
