@@ -191,6 +191,13 @@ WikipediaAppletPrivate::_gotoArtist()
     scheduleEngineUpdate();
 }
 
+void
+WikipediaAppletPrivate::_gotoComposer()
+{
+    dataContainer->setData( "goto", "composer" );
+    scheduleEngineUpdate();
+}
+
 void
 WikipediaAppletPrivate::_gotoTrack()
 {
@@ -613,6 +620,12 @@ WikipediaApplet::init()
     d->artistIcon = addRightHeaderAction( artistAction );
     connect( d->artistIcon, SIGNAL(clicked()), this, SLOT(_gotoArtist()) );
 
+    QAction* composerAction = new QAction( this );
+    composerAction->setIcon( KIcon( "filename-composer-amarok" ) );
+    composerAction->setText( i18n( "Composer" ) );
+    d->composerIcon = addRightHeaderAction( composerAction );
+    connect( d->composerIcon, SIGNAL(clicked()), this, SLOT(_gotoComposer()) );
+
     QAction* albumAction = new QAction( this );
     albumAction->setIcon( KIcon( "filename-album-amarok" ) );
     albumAction->setText( i18n( "Album" ) );
@@ -758,6 +771,9 @@ void
 WikipediaApplet::createConfigurationInterface( KConfigDialog *parent )
 {
     Q_D( WikipediaApplet );
+
+    parent->setButtons( KDialog::Ok | KDialog::Cancel );
+
     KConfigGroup configuration = config();
     QWidget *langSettings = new QWidget;
     d->languageSettingsUi.setupUi( langSettings );
@@ -773,7 +789,6 @@ WikipediaApplet::createConfigurationInterface( KConfigDialog *parent )
     d->generalSettingsUi.mobileCheckBox->setCheckState( d->useMobileWikipedia ? Qt::Checked : Qt::Unchecked );
 
     connect( d->languageSettingsUi.downloadButton, SIGNAL(clicked()), this, SLOT(_getLangMap()) );
-    connect( parent, SIGNAL(applyClicked()), this, SLOT(_loadSettings()) );
     connect( parent, SIGNAL(okClicked()), this, SLOT(_loadSettings()) );
 
     parent->addPage( genSettings, i18n( "Wikipedia General Settings" ), "configure" );
