@@ -16,12 +16,15 @@
  ****************************************************************************************/
 
 #define DEBUG_PREFIX "SqlRegistry"
-#include "core/support/Debug.h"
 
 #include "SqlRegistry.h"
+
+#include "DatabaseUpdater.h"
 #include "SqlRegistry_p.h"
 #include "SqlCollection.h"
-#include "../ScanManager.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/db/MountPointManager.h"
+#include "scanner/GenericScanManager.h"
 
 #include <QMutableHashIterator>
 #include <QMutexLocker>
@@ -39,6 +42,15 @@ SqlRegistry::SqlRegistry( Collections::SqlCollection* collection )
     // we have to do this now before anyone can hold references
     // to those objects.
     DatabaseUpdater databaseUpdater( m_collection );
+
+    // url entries without associated directory just stick around and cannot be processed
+    // by SqlScanResultProcessor. Delete them before checking tracks
+    databaseUpdater.deleteOrphanedByDirectory( "urls" );
+
+    // tracks with no associated url entry are useless, just a bunch of medatada with
+    // nothing to associate them to; remove those first
+    databaseUpdater.deleteOrphanedByUrl( "tracks" );
+
     databaseUpdater.deleteAllRedundant( "album" ); // what about cover images in database and disk cache?
     databaseUpdater.deleteAllRedundant( "artist" );
     databaseUpdater.deleteAllRedundant( "genre" );
@@ -53,7 +65,7 @@ SqlRegistry::SqlRegistry( Collections::SqlCollection* collection )
     m_timer = new QTimer( this );
     m_timer->setInterval( 30 * 1000 );  //try to clean up every 30 seconds, change if necessary
     m_timer->setSingleShot( false );
-    connect( m_timer, SIGNAL( timeout() ), this, SLOT( emptyCache() ) );
+    connect( m_timer, SIGNAL(timeout()), this, SLOT(emptyCache()) );
     m_timer->start();
 }
 
@@ -81,7 +93,7 @@ SqlRegistry::getDirectory( const QString &path, uint mtime )
     // - create new entry
     if( res.isEmpty() )
     {
-        debug() << "SqlRegistry::getDirectory new Directory" << path;
+        debug() << "SqlRegistry::getDirectory(): new directory" << path;
         QString insert = QString( "INSERT INTO directories(deviceid,changedate,dir) "
                                   "VALUES (%1,%2,'%3');" )
                         .arg( QString::number( deviceId ), QString::number( mtime ),
@@ -99,8 +111,8 @@ SqlRegistry::getDirectory( const QString &path, uint mtime )
             QString update = QString( "UPDATE directories SET changedate = %1 "
                                       "WHERE id = %2;" )
                 .arg( QString::number( mtime ), res[0] );
-        debug() << "SqlRegistry::getDirectory update Directory"<<res[0]<<"from" << oldMtime << "to" << mtime;
-        debug() << update;
+            debug() << "SqlRegistry::getDirectory(): update directory" << path << "(id" <<
+                    res[0] << ") from" << oldMtime << "to" << mtime << "UNIX time";
             storage->query( update );
         }
     }
@@ -112,8 +124,6 @@ SqlRegistry::getDirectory( const QString &path, uint mtime )
 Meta::TrackPtr
 SqlRegistry::getTrack( int urlId )
 {
-    QMutexLocker locker( &m_trackMutex );
-
     QString query = "SELECT %1 FROM urls %2 "
         "WHERE urls.id = %3";
     query = query.arg( Meta::SqlTrack::getTrackReturnValues(),
@@ -127,6 +137,7 @@ SqlRegistry::getTrack( int urlId )
                 rowData[Meta::SqlTrack::returnIndex_urlRPath] );
     QString uid = rowData[Meta::SqlTrack::returnIndex_urlUid];
 
+    QMutexLocker locker( &m_trackMutex );
     if( m_trackMap.contains( id ) )
         return m_trackMap.value( id );
     else if( m_uidMap.contains( uid ) )
@@ -147,8 +158,9 @@ SqlRegistry::getTrack( const QString &path )
 {
     int deviceId = m_collection->mountPointManager()->getIdForUrl( path );
     QString rpath = m_collection->mountPointManager()->getRelativePath( deviceId, path );
+    TrackPath id( deviceId, rpath );
 
-    TrackPath id(deviceId, rpath);
+    QMutexLocker locker( &m_trackMutex );
     if( m_trackMap.contains( id ) )
         return m_trackMap.value( id );
     else
@@ -179,7 +191,8 @@ SqlRegistry::getTrack( const QString &path )
 Meta::TrackPtr
 SqlRegistry::getTrack( int deviceId, const QString &rpath, int directoryId, const QString &uidUrl )
 {
-    TrackPath id(deviceId, rpath);
+    TrackPath id( deviceId, rpath );
+
     QMutexLocker locker( &m_trackMutex );
     if( m_trackMap.contains( id ) )
         return m_trackMap.value( id );
@@ -241,15 +254,15 @@ SqlRegistry::getTrack( int trackId, const QStringList &rowData )
 bool
 SqlRegistry::updateCachedUrl( const QString &oldUrl, const QString &newUrl )
 {
-    QMutexLocker locker( &m_trackMutex );
     int deviceId = m_collection->mountPointManager()->getIdForUrl( oldUrl );
     QString rpath = m_collection->mountPointManager()->getRelativePath( deviceId, oldUrl );
-    TrackPath oldId(deviceId, rpath);
+    TrackPath oldId( deviceId, rpath );
 
     int newdeviceId = m_collection->mountPointManager()->getIdForUrl( newUrl );
     QString newRpath = m_collection->mountPointManager()->getRelativePath( newdeviceId, newUrl );
     TrackPath newId( newdeviceId, newRpath );
 
+    QMutexLocker locker( &m_trackMutex );
     if( m_trackMap.contains( newId ) )
         warning() << "updating path to an already existing path.";
     else if( !m_trackMap.contains( oldId ) )
@@ -316,8 +329,6 @@ SqlRegistry::getTrackFromUid( const QString &uid )
 void
 SqlRegistry::removeTrack( int urlId, const QString uid )
 {
-    QMutexLocker locker( &m_trackMutex );
-
     // delete all entries linked to the url, including track
     QStringList tables = QStringList() << "tracks" << "lyrics" << "statistics" << "urls_labels";
     foreach( const QString &table, tables )
@@ -336,6 +347,7 @@ SqlRegistry::removeTrack( int urlId, const QString uid )
     m_collection->sqlStorage()->query( query );
 
     // --- delete the track from memory
+    QMutexLocker locker( &m_trackMutex );
     if( m_uidMap.contains( uid ) )
     {
         // -- remove from hashes
@@ -352,10 +364,11 @@ SqlRegistry::removeTrack( int urlId, const QString uid )
 // -------- artist
 
 Meta::ArtistPtr
-SqlRegistry::getArtist( const QString &name )
+SqlRegistry::getArtist( const QString &oName )
 {
     QMutexLocker locker( &m_artistMutex );
 
+    QString name = oName.left( DatabaseUpdater::textColumnLength() );
     if( m_artistMap.contains( name ) )
         return m_artistMap.value( name );
 
@@ -421,10 +434,11 @@ SqlRegistry::getArtist( int id, const QString &name )
 // -------- genre
 
 Meta::GenrePtr
-SqlRegistry::getGenre( const QString &name )
+SqlRegistry::getGenre( const QString &oName )
 {
     QMutexLocker locker( &m_genreMutex );
 
+    QString name = oName.left( DatabaseUpdater::textColumnLength() );
     if( m_genreMap.contains( name ) )
         return m_genreMap.value( name );
 
@@ -484,10 +498,11 @@ SqlRegistry::getGenre( int id, const QString &name )
 // -------- composer
 
 Meta::ComposerPtr
-SqlRegistry::getComposer( const QString &name )
+SqlRegistry::getComposer( const QString &oName )
 {
     QMutexLocker locker( &m_composerMutex );
 
+    QString name = oName.left( DatabaseUpdater::textColumnLength() );
     if( m_composerMap.contains( name ) )
         return m_composerMap.value( name );
 
@@ -585,14 +600,14 @@ SqlRegistry::getYear( int year, int yearId )
 // -------- album
 
 Meta::AlbumPtr
-SqlRegistry::getAlbum( const QString &name, const QString &artist )
+SqlRegistry::getAlbum( const QString &oName, const QString &oArtist )
 {
-    QString albumArtist( artist );
     // we allow albums with empty name but nonempty artist, see bug 272471
+    QString name = oName.left( DatabaseUpdater::textColumnLength() );
+    QString albumArtist = oArtist.left( DatabaseUpdater::textColumnLength() );
+    AlbumKey key( name, albumArtist );
 
     QMutexLocker locker( &m_albumMutex );
-
-    AlbumKey key(name, albumArtist);
     if( m_albumMap.contains( key ) )
         return m_albumMap.value( key );
 
@@ -647,8 +662,12 @@ SqlRegistry::getAlbum( int albumId )
 {
     Q_ASSERT( albumId > 0 ); // must be a valid id
 
-    if( m_albumIdMap.contains( albumId ) )
-        return m_albumIdMap.value( albumId );
+    {
+        // we want locker only for this block because we call another getAlbum() below
+        QMutexLocker locker( &m_albumMutex );
+        if( m_albumIdMap.contains( albumId ) )
+            return m_albumIdMap.value( albumId );
+    }
 
     QString query = QString( "SELECT name, artist FROM albums WHERE id = %1" ).arg( albumId );
     QStringList res = m_collection->sqlStorage()->query( query );
@@ -664,8 +683,8 @@ Meta::AlbumPtr
 SqlRegistry::getAlbum( int albumId, const QString &name, int artistId )
 {
     Q_ASSERT( albumId > 0 ); // must be a valid id
-    QMutexLocker locker( &m_albumMutex );
 
+    QMutexLocker locker( &m_albumMutex );
     if( m_albumIdMap.contains( albumId ) )
         return m_albumIdMap.value( albumId );
 
@@ -684,10 +703,10 @@ SqlRegistry::getAlbum( int albumId, const QString &name, int artistId )
 // ------------ label
 
 Meta::LabelPtr
-SqlRegistry::getLabel( const QString &label )
+SqlRegistry::getLabel( const QString &oLabel )
 {
     QMutexLocker locker( &m_labelMutex );
-
+    QString label = oLabel.left( DatabaseUpdater::textColumnLength() );
     if( m_labelMap.contains( label ) )
         return m_labelMap.value( label );
 
@@ -752,7 +771,7 @@ void
 SqlRegistry::blockDatabaseUpdate()
 {
     QMutexLocker locker( &m_blockMutex );
-    m_blockDatabaseUpdateCount ++;
+    m_blockDatabaseUpdateCount++;
 }
 
 void
@@ -761,7 +780,7 @@ SqlRegistry::unblockDatabaseUpdate()
     {
         QMutexLocker locker( &m_blockMutex );
         Q_ASSERT( m_blockDatabaseUpdateCount > 0 );
-        m_blockDatabaseUpdateCount --;
+        m_blockDatabaseUpdateCount--;
     }
 
     // update the database
@@ -799,30 +818,38 @@ SqlRegistry::commitDirtyTracks()
     // -- notify all observers
     foreach( Meta::SqlYearPtr year, dirtyYears )
     {
+        // this means that a new year was added to track or an old removed (or both),
+        // Collection docs says we need to emit updated() in this case. Ditto below.
+        m_collectionChanged = true;
         year->invalidateCache();
         year->notifyObservers();
     }
     foreach( Meta::SqlGenrePtr genre, dirtyGenres )
     {
+        m_collectionChanged = true;
         genre->invalidateCache();
         genre->notifyObservers();
     }
     foreach( Meta::SqlAlbumPtr album, dirtyAlbums )
     {
+        m_collectionChanged = true;
         album->invalidateCache();
         album->notifyObservers();
     }
     foreach( Meta::SqlTrackPtr track, dirtyTracks )
     {
+        // if only track changes, no need to emit updated() from here
         track->notifyObservers();
     }
     foreach( Meta::SqlArtistPtr artist, dirtyArtists )
     {
+        m_collectionChanged = true;
         artist->invalidateCache();
         artist->notifyObservers();
     }
     foreach( Meta::SqlComposerPtr composer, dirtyComposers )
     {
+        m_collectionChanged = true;
         composer->invalidateCache();
         composer->notifyObservers();
     }
