@@ -18,14 +18,15 @@
 
 #include "amarokurls/BookmarkMetaActions.h"
 #include "amarokurls/PlayUrlRunner.h"
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core/support/Debug.h"
-#include "core/capabilities/EditCapability.h"
 #include "core/capabilities/ActionsCapability.h"
+#include "core/collections/support/SqlStorage.h"
+#include "core/meta/TrackEditor.h"
+#include "core/support/Debug.h"
 #include "core-impl/capabilities/timecode/TimecodeLoadCapability.h"
 #include "core-impl/capabilities/timecode/TimecodeWriteCapability.h"
-#include "SqlPodcastProvider.h"
-#include "core/collections/support/SqlStorage.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/meta/proxy/MetaProxy.h"
+#include "core-impl/podcasts/sql/SqlPodcastProvider.h"
 
 #include <QDate>
 #include <QFile>
@@ -114,7 +115,6 @@ SqlPodcastEpisode::toPodcastEpisodeList( SqlPodcastEpisodeList episodes )
 
 SqlPodcastEpisode::SqlPodcastEpisode( const QStringList &result, SqlPodcastChannelPtr sqlChannel )
     : Podcasts::PodcastEpisode( Podcasts::PodcastChannelPtr::staticCast( sqlChannel ) )
-    , m_batchUpdate( false )
     , m_channel( sqlChannel )
 {
     SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
@@ -134,19 +134,18 @@ SqlPodcastEpisode::SqlPodcastEpisode( const QStringList &result, SqlPodcastChann
     m_duration = (*(iter++)).toInt();
     m_fileSize = (*(iter++)).toInt();
     m_isNew = sqlStorage->boolTrue() == (*(iter++));
+    m_isKeep = sqlStorage->boolTrue() == (*(iter++));
 
     Q_ASSERT_X( iter == result.constEnd(), "SqlPodcastEpisode( PodcastCollection*, QStringList )", "number of expected fields did not match number of actual fields" );
 
-    if( !m_localUrl.isEmpty() && QFileInfo( m_localUrl.toLocalFile() ).exists() )
-    {
-        m_localFile = new MetaFile::Track( m_localUrl );
-    }
+    setupLocalFile();
 }
 
-// XXX: why do PodcastMetaCommon and PodcastEpisode not have an apropriate copy constructor?
+//TODO: why do PodcastMetaCommon and PodcastEpisode not have an appropriate copy constructor?
 SqlPodcastEpisode::SqlPodcastEpisode( Podcasts::PodcastEpisodePtr episode )
     : Podcasts::PodcastEpisode()
     , m_dbId( 0 )
+    , m_isKeep( false )
 {
     m_channel = SqlPodcastChannelPtr::dynamicCast( episode->channel() );
 
@@ -154,6 +153,7 @@ SqlPodcastEpisode::SqlPodcastEpisode( Podcasts::PodcastEpisodePtr episode )
     {
         debug() << "BUG: creating SqlEpisode but not an sqlChannel!!!";
         debug() <<  episode->channel()->title();
+        debug() <<  m_channel->title();
     }
 
     // PodcastMetaCommon
@@ -163,7 +163,7 @@ SqlPodcastEpisode::SqlPodcastEpisode( Podcasts::PodcastEpisodePtr episode )
     m_subtitle = episode->subtitle();
     m_summary = episode->summary();
     m_author = episode->author();
-    
+
     // PodcastEpisode
     m_guid = episode->guid();
     m_url = KUrl( episode->uidUrl() );
@@ -182,11 +182,64 @@ SqlPodcastEpisode::SqlPodcastEpisode( Podcasts::PodcastEpisodePtr episode )
 
     //commit to the database
     updateInDb();
+    setupLocalFile();
+}
 
-    if( !m_localUrl.isEmpty() && QFileInfo( m_localUrl.toLocalFile() ).exists() )
+SqlPodcastEpisode::SqlPodcastEpisode( PodcastChannelPtr channel, Podcasts::PodcastEpisodePtr episode )
+    : Podcasts::PodcastEpisode()
+    , m_dbId( 0 )
+    , m_isKeep( false )
+{
+    m_channel = SqlPodcastChannelPtr::dynamicCast( channel );
+
+    if( !m_channel && episode->channel() )
     {
-        m_localFile = new MetaFile::Track( m_localUrl );
+        debug() << "BUG: creating SqlEpisode but not an sqlChannel!!!";
+        debug() <<  episode->channel()->title();
+        debug() <<  m_channel->title();
     }
+
+    // PodcastMetaCommon
+    m_title = episode->title();
+    m_description = episode->description();
+    m_keywords = episode->keywords();
+    m_subtitle = episode->subtitle();
+    m_summary = episode->summary();
+    m_author = episode->author();
+
+    // PodcastEpisode
+    m_guid = episode->guid();
+    m_url = KUrl( episode->uidUrl() );
+    m_localUrl = episode->localUrl();
+    m_mimeType = episode->mimeType();
+    m_pubDate = episode->pubDate();
+    m_duration = episode->duration();
+    m_fileSize = episode->filesize();
+    m_sequenceNumber = episode->sequenceNumber();
+    m_isNew = episode->isNew();
+
+    // The album, artist, composer, genre and year fields
+    // contain proxy objects with internal references to this.
+    // These proxies are created by Podcasts::PodcastEpisode(), so
+    // these fields don't have to be set here.
+
+    //commit to the database
+    updateInDb();
+    setupLocalFile();
+}
+
+void
+SqlPodcastEpisode::setupLocalFile()
+{
+    if( m_localUrl.isEmpty() || !QFileInfo( m_localUrl.toLocalFile() ).exists() )
+        return;
+
+    MetaProxy::TrackPtr proxyTrack( new MetaProxy::Track( m_localUrl, MetaProxy::Track::ManualLookup ) );
+    m_localFile = Meta::TrackPtr( proxyTrack.data() ); // avoid static_cast
+    /* following won't write to actual file, because MetaProxy::Track hasn't yet looked
+     * up the underlying track. It will just set some cached values. */
+    writeTagsToFile();
+    proxyTrack->lookupTrack( CollectionManager::instance()->fileTrackProvider() );
 }
 
 SqlPodcastEpisode::~SqlPodcastEpisode()
@@ -196,7 +249,13 @@ SqlPodcastEpisode::~SqlPodcastEpisode()
 void
 SqlPodcastEpisode::setNew( bool isNew )
 {
-    m_isNew = isNew;
+    PodcastEpisode::setNew( isNew );
+    updateInDb();
+}
+
+void SqlPodcastEpisode::setKeep( bool isKeep )
+{
+    m_isKeep = isKeep;
     updateInDb();
 }
 
@@ -241,10 +300,6 @@ SqlPodcastEpisode::hasCapabilityInterface( Capabilities::Capability::Type type )
             //only downloaded episodes can be position marked
 //            return !localUrl().isEmpty();
             return true;
-            //TODO: downloaded episodes can be edited
-        case Capabilities::Capability::Editable:
-            return isEditable();
-
         default:
             return false;
     }
@@ -265,23 +320,11 @@ SqlPodcastEpisode::createCapabilityInterface( Capabilities::Capability::Type typ
             return new TimecodeWriteCapabilityPodcastImpl( this );
         case Capabilities::Capability::LoadTimecode:
             return new TimecodeLoadCapabilityPodcastImpl( this );
-        case Capabilities::Capability::Editable:
-            if( !m_localFile.isNull() )
-                return m_localFile->createCapabilityInterface( type );
         default:
             return 0;
     }
 }
 
-bool
-SqlPodcastEpisode::isEditable() const
-{
-     if( m_localFile.isNull() )
-         return false;
-
-     return m_localFile->isEditable();
-}
-
 void
 SqlPodcastEpisode::finishedPlaying( double playedFraction )
 {
@@ -312,12 +355,11 @@ SqlPodcastEpisode::prettyName() const
 void
 SqlPodcastEpisode::setTitle( const QString &title )
 {
-    if( !m_localFile.isNull() )
-    {
-        m_localFile->setTitle( title );
-    }
-
     m_title = title;
+
+    Meta::TrackEditorPtr ec = m_localFile ? m_localFile->editor() : Meta::TrackEditorPtr();
+    if( ec  )
+        ec->setTitle( title );
 }
 
 Meta::ArtistPtr
@@ -356,32 +398,35 @@ SqlPodcastEpisode::year() const
     return m_localFile->year();
 }
 
+Meta::TrackEditorPtr
+SqlPodcastEpisode::editor()
+{
+    if( m_localFile )
+        return m_localFile->editor();
+    else
+        return Meta::TrackEditorPtr();
+}
+
 bool
 SqlPodcastEpisode::writeTagsToFile()
 {
-    if( m_localFile.isNull() )
+    if( !m_localFile )
         return false;
 
-    QScopedPointer<Capabilities::EditCapability> ec( m_localFile->create<Capabilities::EditCapability>() );
+    Meta::TrackEditorPtr ec = m_localFile->editor();
     if( !ec )
         return false;
 
     debug() << "writing tags for podcast episode " << title() << "to " << m_localUrl.url();
-    if( !ec->isEditable() )
-    {
-        debug() << QString( "local file (%1)is not editable!" ).arg( m_localUrl.url() );
-        return false;
-    }
-    ec->beginMetaDataUpdate();
+    ec->beginUpdate();
     ec->setTitle( m_title );
     ec->setAlbum( m_channel->title() );
     ec->setArtist( m_channel->author() );
     ec->setGenre( i18n( "Podcast" ) );
     ec->setYear( m_pubDate.date().year() );
-    ec->endMetaDataUpdate();
+    ec->endUpdate();
 
     notifyObservers();
-
     return true;
 }
 
@@ -423,7 +468,9 @@ SqlPodcastEpisode::updateInDb()
         stream << ", filesize=";
         stream << m_fileSize;
         stream << ", isnew=";
-        stream << (m_isNew ? boolTrue : boolFalse);
+        stream << (isNew() ? boolTrue : boolFalse);
+        stream << ", iskeep=";
+        stream << (isKeep() ? boolTrue : boolFalse);
         stream << " WHERE id=";
         stream << m_dbId;
         stream << ";";
@@ -433,7 +480,7 @@ SqlPodcastEpisode::updateInDb()
     {
         stream << "INSERT INTO podcastepisodes (";
         stream << "url,channel,localurl,guid,title,subtitle,sequencenumber,description,";
-        stream << "mimetype,pubdate,duration,filesize,isnew) ";
+        stream << "mimetype,pubdate,duration,filesize,isnew,iskeep) ";
         stream << "VALUES ( '";
         stream << escape(m_url.url()) << "', ";
         stream << m_channel->dbId() << ", '";
@@ -447,7 +494,8 @@ SqlPodcastEpisode::updateInDb()
         stream << escape(m_pubDate.toString(Qt::ISODate)) << "', ";
         stream << m_duration << ", ";
         stream << m_fileSize << ", ";
-        stream << (m_isNew ? boolTrue : boolFalse);
+        stream << (isNew() ? boolTrue : boolFalse) << ", ";
+        stream << (isKeep() ? boolTrue : boolFalse);
         stream << ");";
         m_dbId = sqlStorage->insert( command, "podcastepisodes" );
     }
@@ -498,6 +546,7 @@ SqlPodcastChannel::SqlPodcastChannel( SqlPodcastProvider *provider,
     m_purge = sqlStorage->boolTrue() == *(iter++);
     m_purgeCount = (*(iter++)).toInt();
     m_writeTags = sqlStorage->boolTrue() == *(iter++);
+    m_filenameLayout = *(iter++);
 }
 
 SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
@@ -505,6 +554,7 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
     : Podcasts::PodcastChannel()
     , m_dbId( 0 )
     , m_provider( provider )
+    , m_filenameLayout( "%default%" )
 {
     // PodcastMetaCommon
     m_title = channel->title();
@@ -513,7 +563,7 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
     m_subtitle = channel->subtitle();
     m_summary = channel->summary();
     m_author = channel->author();
-    
+
     // PodcastChannel
     m_url = channel->url();
     m_webLink = channel->webLink();
@@ -521,9 +571,9 @@ SqlPodcastChannel::SqlPodcastChannel( Podcasts::SqlPodcastProvider *provider,
     m_labels = channel->labels();
     m_subscribeDate = channel->subscribeDate();
     m_copyright = channel->copyright();
-    
+
     if( channel->hasImage() )
-        m_image = channel->image().toImage();
+        m_image = channel->image();
 
     //Default Settings
 
@@ -552,20 +602,8 @@ SqlPodcastChannel::trackCount() const
 {
     if( m_episodesLoaded )
         return m_episodes.count();
-
-    QString query = "SELECT COUNT(id) FROM podcastepisodes WHERE channel = %1";
-
-    SqlStorage *sql = CollectionManager::instance()->sqlStorage();
-    Q_ASSERT( sql );
-
-    QStringList results = sql->query( query.arg( m_dbId ) );
-    if( results.isEmpty() )
-    {
-        error() << "no results for COUNT query on playlist_tracks table!";
+    else
         return -1;
-    }
-    int trackCount = results.first().toInt();
-    return m_purge ? qMin( m_purgeCount, trackCount ): trackCount;
 }
 
 void
@@ -573,6 +611,7 @@ SqlPodcastChannel::triggerTrackLoad()
 {
     if( !m_episodesLoaded )
         loadEpisodes();
+    notifyObserversTracksLoaded();
 }
 
 Playlists::PlaylistProvider *
@@ -615,7 +654,7 @@ SqlPodcastChannel::setTitle( const QString &title )
 }
 
 Podcasts::PodcastEpisodeList
-SqlPodcastChannel::episodes()
+SqlPodcastChannel::episodes() const
 {
     return SqlPodcastEpisode::toPodcastEpisodeList( m_episodes );
 }
@@ -647,21 +686,32 @@ SqlPodcastChannel::setImageUrl( const KUrl &imageUrl )
 Podcasts::PodcastEpisodePtr
 SqlPodcastChannel::addEpisode( PodcastEpisodePtr episode )
 {
-    DEBUG_BLOCK
-    debug() << "adding episode " << episode->title() << " to sqlchannel " << title();
-
     if( !m_provider )
         return PodcastEpisodePtr();
 
-    //check for guid and return the episode we might already have.
-    if( !episode->guid().isEmpty() && m_provider->possiblyContainsTrack( episode->guid() ) )
+    KUrl checkUrl;
+    //searched in the database for guid or enclosure url
+    if( !episode->guid().isEmpty() )
+        checkUrl = episode->guid();
+    else if( !episode->uidUrl().isEmpty() )
+        checkUrl = episode->uidUrl();
+    else
+        return PodcastEpisodePtr(); //noting to check for
+
+    if( m_provider->possiblyContainsTrack( checkUrl ) )
         return PodcastEpisodePtr::dynamicCast( m_provider->trackForUrl( episode->guid() ) );
 
     //force episodes load.
     if( !m_episodesLoaded )
         loadEpisodes();
 
-    SqlPodcastEpisodePtr sqlEpisode = SqlPodcastEpisodePtr( new SqlPodcastEpisode( episode ) );
+    SqlPodcastEpisodePtr sqlEpisode;
+
+    if (SqlPodcastEpisodePtr::dynamicCast( episode ))
+        sqlEpisode = SqlPodcastEpisodePtr( new SqlPodcastEpisode( episode ) );
+    else
+        sqlEpisode = SqlPodcastEpisodePtr( new SqlPodcastEpisode( PodcastChannelPtr(this) , episode ) );
+
 
     //episodes are sorted on pubDate high to low
     int i;
@@ -688,18 +738,28 @@ SqlPodcastChannel::addEpisode( PodcastEpisodePtr episode )
 void
 SqlPodcastChannel::applyPurge()
 {
+    DEBUG_BLOCK
     if( !hasPurge() )
         return;
 
-    while( m_episodes.count() > purgeCount() )
+    if( m_episodes.count() > purgeCount() )
     {
-        SqlPodcastEpisodePtr removedEpisode = m_episodes.takeLast();
-        m_provider->deleteDownloadedEpisode( removedEpisode );
+        int purgeIndex = 0;
 
-        notifyObserversTrackRemoved( m_episodes.count() );
+        foreach( SqlPodcastEpisodePtr episode, m_episodes )
+        {
+            if ( !episode->isKeep() )
+            {
+                if( purgeIndex >= purgeCount() )
+                {
+                    m_provider->deleteDownloadedEpisode( episode );
+                    m_episodes.removeOne( episode );
+                }
+                else
+                    purgeIndex++;
+            }
+        }
     }
-
-    //TODO: load missing episodes in case m_episodes.count() <= purgeCount()
 }
 
 void
@@ -712,13 +772,13 @@ SqlPodcastChannel::updateInDb()
     #define escape(x) sqlStorage->escape(x)
     QString insert = "INSERT INTO podcastchannels("
     "url,title,weblink,image,description,copyright,directory,labels,"
-    "subscribedate,autoscan,fetchtype,haspurge,purgecount,writetags) "
-    "VALUES ( '%1','%2','%3','%4','%5','%6','%7','%8','%9',%10,%11,%12,%13,%14 );";
+    "subscribedate,autoscan,fetchtype,haspurge,purgecount,writetags,filenamelayout) "
+    "VALUES ( '%1','%2','%3','%4','%5','%6','%7','%8','%9',%10,%11,%12,%13,%14,'%15' );";
 
     QString update = "UPDATE podcastchannels SET url='%1',title='%2'"
     ",weblink='%3',image='%4',description='%5',copyright='%6',directory='%7'"
     ",labels='%8',subscribedate='%9',autoscan=%10,fetchtype=%11,haspurge=%12,"
-    "purgecount=%13, writetags=%14 WHERE id=%15;";
+    "purgecount=%13, writetags=%14, filenamelayout='%15' WHERE id=%16;";
     //if we don't have a database ID yet we should insert;
     QString command = m_dbId ? update : insert;
 
@@ -736,6 +796,7 @@ SqlPodcastChannel::updateInDb()
     command = command.arg( m_purge ? boolTrue : boolFalse ); //%12
     command = command.arg( QString::number(m_purgeCount) ); //%13
     command = command.arg( m_writeTags ? boolTrue : boolFalse ); //%14
+    command = command.arg( escape(m_filenameLayout) ); //%15
 
     if( m_dbId )
         sqlStorage->query( command.arg( m_dbId ) );
@@ -764,34 +825,78 @@ SqlPodcastChannel::loadEpisodes()
 
     SqlStorage *sqlStorage = CollectionManager::instance()->sqlStorage();
 
-    //if purge is enabled is true we limit the number of results
+    //If purge is enabled we must limit the number of results
     QString command;
+
+    int rowLength = 15;
+
+    //If purge is enabled we must limit the number of results, though there are some files
+    //the user want to be shown even if there is no more slot
     if( hasPurge() )
     {
-        command = QString( "SELECT id, url, channel, localurl, guid, "
-        "title, subtitle, sequencenumber, description, mimetype, pubdate, "
-        "duration, filesize, isnew FROM podcastepisodes WHERE channel = %1 "
-        "ORDER BY pubdate DESC LIMIT " + QString::number( purgeCount() ) + ';' );
+        command = QString( "(SELECT id, url, channel, localurl, guid, "
+                           "title, subtitle, sequencenumber, description, mimetype, pubdate, "
+                           "duration, filesize, isnew, iskeep FROM podcastepisodes WHERE channel = %1 "
+                           "AND iskeep IS FALSE ORDER BY pubdate DESC LIMIT " + QString::number( purgeCount() ) + ") "
+                           "UNION "
+                           "(SELECT id, url, channel, localurl, guid, "
+                           "title, subtitle, sequencenumber, description, mimetype, pubdate, "
+                           "duration, filesize, isnew, iskeep FROM podcastepisodes WHERE channel = %1 "
+                           "AND iskeep IS TRUE) "
+                           "ORDER BY pubdate DESC;"
+                           );
     }
     else
     {
         command = QString( "SELECT id, url, channel, localurl, guid, "
-            "title, subtitle, sequencenumber, description, mimetype, pubdate, "
-            "duration, filesize, isnew FROM podcastepisodes WHERE channel = %1 "
-            "ORDER BY pubdate DESC;" );
+                           "title, subtitle, sequencenumber, description, mimetype, pubdate, "
+                           "duration, filesize, isnew, iskeep FROM podcastepisodes WHERE channel = %1 "
+                           "ORDER BY pubdate DESC;"
+                           );
     }
 
     QStringList results = sqlStorage->query( command.arg( m_dbId ) );
 
-    int rowLength = 14;
-    for(int i=0; i < results.size(); i+=rowLength)
+    for( int i = 0; i < results.size(); i += rowLength )
     {
         QStringList episodesResult = results.mid( i, rowLength );
         SqlPodcastEpisodePtr sqlEpisode = SqlPodcastEpisodePtr(
-                new SqlPodcastEpisode( episodesResult, SqlPodcastChannelPtr( this ) ) );
-        m_episodes <<  sqlEpisode;
+                                              new SqlPodcastEpisode(
+                                                  episodesResult,
+                                                  SqlPodcastChannelPtr( this ) ) );
+        m_episodes << sqlEpisode;
     }
 
     m_episodesLoaded = true;
 }
 
+Meta::TrackList
+Podcasts::SqlPodcastChannel::tracks()
+{
+    return Podcasts::SqlPodcastEpisode::toTrackList( m_episodes );
+}
+
+void
+Podcasts::SqlPodcastChannel::syncTrackStatus( int position, Meta::TrackPtr otherTrack )
+{
+    Q_UNUSED( position );
+
+    Podcasts::PodcastEpisodePtr master =
+            Podcasts::PodcastEpisodePtr::dynamicCast( otherTrack );
+
+    if ( master )
+    {
+        this->setName( master->channel()->name() );
+        this->setTitle( master->channel()->title() );
+        this->setUrl( master->channel()->url() );
+    }
+}
+
+void
+Podcasts::SqlPodcastChannel::addTrack( Meta::TrackPtr track, int position )
+{
+    Q_UNUSED( position );
+
+    addEpisode( Podcasts::PodcastEpisodePtr::dynamicCast( track ) );
+    notifyObserversTrackAdded( track, position );
+}
