@@ -4,6 +4,7 @@
  * Copyright (c) 2004-2008 Mark Kretschmann <kretschmann@kde.org>                       *
  * Copyright (c) 2008 Seb Ruiz <ruiz@kde.org>                                           *
  * Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>                                   *
+ * Copyright (c) 2013 Ralf Engels <ralf-engels@gmx.de>                                  *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -20,10 +21,9 @@
 
 #include "CollectionSetup.h"
 
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core-impl/collections/db/sql/SqlCollection.h"
-#include "core/support/Debug.h"
 #include "amarokconfig.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/support/CollectionManager.h"
 #include "dialogs/DatabaseImporterDialog.h"
 
 #include <KLocale>
@@ -33,152 +33,73 @@
 
 #include <QAction>
 #include <QApplication>
+#include <QCheckBox>
 #include <QDir>
 #include <QFile>
 #include <QLabel>
 #include <QMenu>
 
-CollectionSetupTreeView::CollectionSetupTreeView( QWidget *parent )
-        : QTreeView( parent )
-{
-    DEBUG_BLOCK
-    m_rescanDirAction = new QAction( this );
-    connect( this, SIGNAL( pressed(const QModelIndex &) ), this, SLOT( slotPressed(const QModelIndex&) ) );
-    connect( m_rescanDirAction, SIGNAL( triggered() ), this, SLOT( slotRescanDirTriggered() ) );
-}
-
-CollectionSetupTreeView::~CollectionSetupTreeView()
-{
-    this->disconnect();
-    delete m_rescanDirAction;
-}
-
-void
-CollectionSetupTreeView::slotPressed( const QModelIndex &index )
-{
-    DEBUG_BLOCK
-    // --- show context menu on right mouse button
-    if( ( QApplication::mouseButtons() & Qt::RightButton ) && parent() )
-    {
-        m_currDir = qobject_cast<CollectionSetup*>(parent())->modelFilePath( index );
-        debug() << "Setting current dir to " << m_currDir;
-
-        // check if there is an sql collection covering the directory
-        bool covered = false;
-        QList<Collections::Collection*> queryableCollections = CollectionManager::instance()->queryableCollections();
-        foreach( Collections::Collection *collection, queryableCollections )
-        {
-            if( collection->isDirInCollection( m_currDir ) )
-                covered = true;
-        }
-
-        // it's covered, so we can show the rescan option
-        if( covered )
-        {
-            m_rescanDirAction->setText( i18n( "Rescan '%1'", m_currDir ) );
-            QMenu menu;
-            menu.addAction( m_rescanDirAction );
-            menu.exec( QCursor::pos() );
-        }
-    }
-}
-
-void
-CollectionSetupTreeView::slotRescanDirTriggered()
-{
-    DEBUG_BLOCK
-    CollectionManager::instance()->startIncrementalScan( m_currDir );
-}
-
 CollectionSetup* CollectionSetup::s_instance;
 
 
 CollectionSetup::CollectionSetup( QWidget *parent )
-        : KVBox( parent )
+        : QWidget( parent )
+        , m_rescanDirAction( new QAction( this ) )
 {
-    DEBUG_BLOCK
+    m_ui.setupUi(this);
 
     setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
 
     setObjectName( "CollectionSetup" );
     s_instance = this;
 
-    QLabel* descriptionLabel = new QLabel( i18n(
-        "These folders will be scanned for "
-        "media to make up your collection. You can "
-        "right-click on a folder to individually "
-        "rescan it, if it was previously selected:" ), this );
-    descriptionLabel->setAlignment( Qt::AlignJustify );
-    descriptionLabel->setWordWrap( true );
-
-    m_view  = new CollectionSetupTreeView( this );
-    m_view->setHeaderHidden( true );
-    m_view->setRootIsDecorated( true );
     if( KGlobalSettings::graphicEffectsLevel() != KGlobalSettings::NoEffects )
-        m_view->setAnimated( true );
-    m_view->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
-    connect( m_view, SIGNAL( clicked( const QModelIndex & ) ), this, SIGNAL( changed() ) );
+        m_ui.view->setAnimated( true );
+    connect( m_ui.view, SIGNAL(clicked(QModelIndex)),
+             this, SIGNAL(changed()) );
 
-    KHBox* buttonBox = new KHBox( this );
+    connect( m_ui.view, SIGNAL(pressed(QModelIndex)),
+             this, SLOT(slotPressed(QModelIndex)) );
+    connect( m_rescanDirAction, SIGNAL(triggered()),
+             this, SLOT(slotRescanDirTriggered()) );
 
-    KPushButton *rescan = new KPushButton( KIcon( "collection-rescan-amarok" ), i18n( "Full rescan" ), buttonBox );
+    KPushButton *rescan = new KPushButton( KIcon( "collection-rescan-amarok" ), i18n( "Full rescan" ), m_ui.buttonContainer );
     rescan->setToolTip( i18n( "Rescan your entire collection. This will <i>not</i> delete any statistics." ) );
-    connect( rescan, SIGNAL( clicked() ), CollectionManager::instance(), SLOT( startFullScan() ) );
+    connect( rescan, SIGNAL(clicked()), CollectionManager::instance(), SLOT(startFullScan()) );
 
-    KPushButton *import = new KPushButton( KIcon( "tools-wizard" ), i18n( "Import" ), buttonBox );
+    KPushButton *import = new KPushButton( KIcon( "tools-wizard" ), i18n( "Import" ), m_ui.buttonContainer );
     import->setToolTip( i18n( "Import collection and/or statistics from older Amarok versions, the batch scanner or media players." ) );
-    connect( import, SIGNAL( clicked() ), this, SLOT( importCollection() ) );
+    connect( import, SIGNAL(clicked()), this, SLOT(importCollection()) );
 
-    m_recursive = new QCheckBox( i18n("&Scan folders recursively (requires full rescan if newly checked)"), this );
-    m_monitor   = new QCheckBox( i18n("&Watch folders for changes"), this );
-    m_writeBack = new QCheckBox( i18n("Write metadata to file"), this );
-    m_writeBackStatistics = new QCheckBox( i18n("Write statistics to file"), this );
-    KHBox* writeBackCoverDimensionsBox = new KHBox( this );
-    m_writeBackCover = new QCheckBox( i18n("Write covers to file, maximum size:"), writeBackCoverDimensionsBox );
-    m_writeBackCoverDimensions = new KComboBox( writeBackCoverDimensionsBox );
-    m_writeBackCoverDimensions->addItem( i18nc("Maximum cover size option","Small (200 px)"), QVariant(200) );
-    m_writeBackCoverDimensions->addItem( i18nc("Maximum cover size option","Medium (400 px)"), QVariant(400) );
-    m_writeBackCoverDimensions->addItem( i18nc("Maximum cover size option","Large (800 px)"), QVariant(800) );
-    m_writeBackCoverDimensions->addItem( i18nc("Maximum cover size option","Huge (1600 px)"), QVariant(1600) );
-    m_charset   = new QCheckBox( i18n("&Enable character set detection in ID3 tags"), this );
-    connect( m_recursive, SIGNAL( toggled( bool ) ), this, SIGNAL( changed() ) );
-    connect( m_monitor  , SIGNAL( toggled( bool ) ), this, SIGNAL( changed() ) );
-    connect( m_writeBack, SIGNAL( toggled( bool ) ), this, SIGNAL( changed() ) );
-    connect( m_writeBackStatistics, SIGNAL( toggled( bool ) ), this, SIGNAL( changed() ) );
-    connect( m_writeBackCover, SIGNAL( toggled( bool ) ), this, SIGNAL( changed() ) );
-    connect( m_writeBackCoverDimensions, SIGNAL( currentIndexChanged( int )), this, SIGNAL( changed() ) );
-    connect( m_charset  , SIGNAL( toggled( bool ) ), this, SIGNAL( changed() ) );
+    QHBoxLayout *buttonLayout = new QHBoxLayout();
+    buttonLayout->addWidget( rescan );
+    buttonLayout->addWidget( import );
+    m_ui.buttonContainer->setLayout( buttonLayout );
+
+    m_recursive = new QCheckBox( i18n("&Scan folders recursively (requires full rescan if newly checked)"), m_ui.checkboxContainer );
+    m_monitor   = new QCheckBox( i18n("&Watch folders for changes"), m_ui.checkboxContainer );
+    connect( m_recursive, SIGNAL(toggled(bool)), this, SIGNAL(changed()) );
+    connect( m_monitor  , SIGNAL(toggled(bool)), this, SIGNAL(changed()) );
+
+    QVBoxLayout *checkboxLayout = new QVBoxLayout();
+    checkboxLayout->addWidget( m_recursive );
+    checkboxLayout->addWidget( m_monitor );
+    m_ui.checkboxContainer->setLayout( checkboxLayout );
 
     m_recursive->setToolTip( i18n( "If selected, Amarok will read all subfolders." ) );
-    m_monitor->setToolTip(   i18n( "If selected, the collection folders will be watched for changes.\nThe watcher will not notice changes behind symbolic links." ) );
-    m_writeBack->setToolTip( i18n( "Write meta data changes (including 'stars' rating) back to the original file.\nYou can also prevent writing back by write protecting the file.\nThis might be a good idea if you are currently\nsharing those files via the Internet." ) );
-    m_writeBackStatistics->setToolTip( i18n( "Write play-changing statistics (e.g. score, lastplayed, playcount)\nas tags back to the file." ) );
-    m_writeBackCover->setToolTip( i18n( "Write changed covers back to the file.\nThis will replace existing embedded covers." ) );
-    m_writeBackCoverDimensions->setToolTip( i18n( "Scale covers down if necessary." ) );
-    m_charset->setToolTip(   i18n( "If selected, Amarok will use Mozilla's\nCharacter Set Detector to attempt to automatically guess the\ncharacter sets used in ID3 tags." ) );
+    m_monitor->setToolTip( i18n( "If selected, the collection folders will be watched "
+            "for changes.\nThe watcher will not notice changes behind symbolic links." ) );
 
     m_recursive->setChecked( AmarokConfig::scanRecursively() );
     m_monitor->setChecked( AmarokConfig::monitorChanges() );
-    m_writeBack->setChecked( AmarokConfig::writeBack() );
-    m_writeBack->setVisible( false ); // probably not a usecase
-    m_writeBackStatistics->setChecked( AmarokConfig::writeBackStatistics() );
-    m_writeBackStatistics->setEnabled( writeBack() );
-    m_writeBackCover->setChecked( AmarokConfig::writeBackCover() );
-    m_writeBackCover->setEnabled( writeBack() );
-    if( m_writeBackCoverDimensions->findData( AmarokConfig::writeBackCoverDimensions() ) != -1 )
-        m_writeBackCoverDimensions->setCurrentIndex( m_writeBackCoverDimensions->findData( AmarokConfig::writeBackCoverDimensions() ) );
-    else
-        m_writeBackCoverDimensions->setCurrentIndex( 1 );
-    m_writeBackCoverDimensions->setEnabled( m_writeBackCover->isEnabled() && m_writeBackCover->isChecked() );
-    m_charset->setChecked( AmarokConfig::useCharsetDetector() );
 
     // set the model _after_ constructing the checkboxes
-    m_model = new CollectionFolder::Model();
-    m_view->setModel( m_model );
+    m_model = new CollectionFolder::Model( this );
+    m_ui.view->setModel( m_model );
     #ifndef Q_OS_WIN
-    m_view->setRootIndex( m_model->setRootPath( QDir::rootPath() ) );
+    m_ui.view->setRootIndex( m_model->setRootPath( QDir::rootPath() ) );
     #else
-    m_view->setRootIndex( m_model->setRootPath( m_model->myComputer().toString() ) );
+    m_ui.view->setRootIndex( m_model->setRootPath( m_model->myComputer().toString() ) );
     #endif
 
     Collections::Collection *primaryCollection = CollectionManager::instance()->primaryCollection();
@@ -189,31 +110,8 @@ CollectionSetup::CollectionSetup( QWidget *parent )
     foreach( const QString &dir, dirs )
     {
         QModelIndex index = m_model->index( dir );
-        m_view->scrollTo( index, QAbstractItemView::EnsureVisible );
+        m_ui.view->scrollTo( index, QAbstractItemView::EnsureVisible );
     }
-
-    setSpacing( 6 );
-}
-
-bool
-CollectionSetup::hasChanged() const
-{
-    Collections::Collection *primaryCollection = CollectionManager::instance()->primaryCollection();
-    QStringList collectionFolders = primaryCollection ? primaryCollection->property( "collectionFolders" ).toStringList() : QStringList();
-
-    m_writeBackStatistics->setEnabled( writeBack() );
-    m_writeBackCover->setEnabled( writeBack() );
-    m_writeBackCoverDimensions->setEnabled( m_writeBackCover->isEnabled() && m_writeBackCover->isChecked() );
-
-    return
-        m_model->directories() != collectionFolders ||
-        m_recursive->isChecked() != AmarokConfig::scanRecursively() ||
-        m_monitor->isChecked() != AmarokConfig::monitorChanges() ||
-        m_writeBack->isChecked() != AmarokConfig::writeBack() ||
-        m_writeBackStatistics->isChecked() != AmarokConfig::writeBackStatistics() ||
-        m_writeBackCover->isChecked() != AmarokConfig::writeBackCover() ||
-        m_writeBackCoverDimensions->itemData(m_writeBackCoverDimensions->currentIndex()).toInt() != AmarokConfig::writeBackCoverDimensions() ||
-        m_charset->isChecked() != AmarokConfig::useCharsetDetector();
 }
 
 void
@@ -223,12 +121,6 @@ CollectionSetup::writeConfig()
 
     AmarokConfig::setScanRecursively( recursive() );
     AmarokConfig::setMonitorChanges( monitor() );
-    AmarokConfig::setWriteBack( writeBack() );
-    AmarokConfig::setWriteBackStatistics( writeBackStatistics() );
-    AmarokConfig::setWriteBackCover( writeBackCover() );
-    if( writeBackCoverDimensions() > 0 )
-        AmarokConfig::setWriteBackCoverDimensions( writeBackCoverDimensions() );
-    AmarokConfig::setUseCharsetDetector( charset() );
 
     Collections::Collection *primaryCollection = CollectionManager::instance()->primaryCollection();
     QStringList collectionFolders = primaryCollection ? primaryCollection->property( "collectionFolders" ).toStringList() : QStringList();
@@ -239,18 +131,31 @@ CollectionSetup::writeConfig()
         if( primaryCollection )
             primaryCollection->setProperty( "collectionFolders", m_model->directories() );
 
-        debug() << "MountPointManager collection folders: " << collectionFolders;
+        debug() << "Old collection folders:      " << collectionFolders;
         CollectionManager::instance()->startFullScan();
     }
 }
 
-void
-CollectionSetup::importCollection()
+bool
+CollectionSetup::hasChanged() const
 {
-    DatabaseImporterDialog *dlg = new DatabaseImporterDialog( this );
-    dlg->exec(); // be modal to avoid messing about by the user in the application
+    Collections::Collection *primaryCollection = CollectionManager::instance()->primaryCollection();
+    QStringList collectionFolders = primaryCollection ? primaryCollection->property( "collectionFolders" ).toStringList() : QStringList();
+
+    return
+        m_model->directories() != collectionFolders ||
+        m_recursive->isChecked() != AmarokConfig::scanRecursively() ||
+        m_monitor->isChecked() != AmarokConfig::monitorChanges();
 }
 
+bool
+CollectionSetup::recursive() const
+{ return m_recursive && m_recursive->isChecked(); }
+
+bool
+CollectionSetup::monitor() const
+{ return m_monitor && m_monitor->isChecked(); }
+
 const QString
 CollectionSetup::modelFilePath( const QModelIndex &index ) const
 {
@@ -258,14 +163,82 @@ CollectionSetup::modelFilePath( const QModelIndex &index ) const
 }
 
 
+void
+CollectionSetup::importCollection()
+{
+    DatabaseImporterDialog *dlg = new DatabaseImporterDialog( this );
+    dlg->exec(); // be modal to avoid messing about by the user in the application
+}
+
+void
+CollectionSetup::slotPressed( const QModelIndex &index )
+{
+    DEBUG_BLOCK
+
+    // --- show context menu on right mouse button
+    if( ( QApplication::mouseButtons() & Qt::RightButton ) )
+    {
+        m_currDir = modelFilePath( index );
+        debug() << "Setting current dir to " << m_currDir;
+
+        // check if there is an sql collection covering the directory
+        // it's covered, so we can show the rescan option
+        if( isDirInCollection( m_currDir ) )
+        {
+            m_rescanDirAction->setText( i18n( "Rescan '%1'", m_currDir ) );
+            QMenu menu;
+            menu.addAction( m_rescanDirAction );
+            menu.exec( QCursor::pos() );
+        }
+    }
+}
+
+void
+CollectionSetup::slotRescanDirTriggered()
+{
+    DEBUG_BLOCK
+    CollectionManager::instance()->startIncrementalScan( m_currDir );
+}
+
+
+bool
+CollectionSetup::isDirInCollection( const QString& path ) const
+{
+    DEBUG_BLOCK
+
+    Collections::Collection *primaryCollection = CollectionManager::instance()->primaryCollection();
+    QStringList collectionFolders = primaryCollection ? primaryCollection->property( "collectionFolders" ).toStringList() : QStringList();
+
+    KUrl url = KUrl( path );
+    KUrl parentUrl;
+    foreach( const QString &dir, collectionFolders )
+    {
+        debug() << "Collection Location: " << dir;
+        debug() << "path: " << path;
+        debug() << "scan Recursively: " << AmarokConfig::scanRecursively();
+        parentUrl.setPath( dir );
+        if ( !AmarokConfig::scanRecursively() )
+        {
+            if ( ( dir == path ) || ( QString( dir + '/' ) == path ) )
+                return true;
+        }
+        else //scan recursively
+        {
+            if ( parentUrl.isParentOf( path ) )
+                return true;
+        }
+    }
+    return false;
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////
 // CLASS Model
 //////////////////////////////////////////////////////////////////////////////////////////
 
 namespace CollectionFolder {
 
-    Model::Model()
-        : QFileSystemModel()
+    Model::Model( QObject *parent )
+        : QFileSystemModel( parent )
     {
         setFilter( QDir::AllDirs | QDir::NoDotAndDotDot );
     }
@@ -440,12 +413,6 @@ namespace CollectionFolder {
         return false;
     }
 
-    /**
-     * Check the logical recursive difference of root and excludePath.
-     * For example, if excludePath is a grandchild of root, then this method
-     * will check all of the children of root except the one that is the
-     * parent of excludePath, as well as excludePath's siblings.
-     */
     void
     Model::checkRecursiveSubfolders( const QString &root, const QString &excludePath )
     {
@@ -467,4 +434,3 @@ namespace CollectionFolder {
     }
 
 } //namespace Collection
-
