@@ -14,7 +14,8 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
 #include "App.h"
 #include "aboutdialog/OcsData.h"
 
@@ -22,55 +23,114 @@
 #include <KCmdLineArgs>
 #include <KDebug>
 
+#include <qglobal.h>
+
+#ifdef Q_WS_X11
+    #include <X11/Xlib.h>
+#endif
+
+#include <csignal>
+
 //#define AMAROK_USE_DRKONQI
-
-extern AMAROK_EXPORT class KAboutData aboutData; //defined in App.cpp
-extern AMAROK_EXPORT class OcsData ocsData;
-
+#ifdef Q_OS_WIN
+AMAROK_EXPORT OcsData ocsData;
+#endif
 
 int main( int argc, char *argv[] )
 {
+    KAboutData aboutData(
+        "amarok", 0,
+        ki18n( "Amarok" ), AMAROK_VERSION,
+        ki18n( "The audio player for KDE" ), KAboutData::License_GPL,
+        ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2012, The Amarok Development Squad" ),
+        ki18n( "IRC:\nirc.freenode.net - #amarok, #amarok.de, #amarok.es, #amarok.fr\n\nFeedback:\namarok@kde.org\n\n(Build Date: %1)" ).subs( __DATE__ ),
+        ( "http://amarok.kde.org" ) );
 
-    //Authors
-    aboutData.addAuthor( ki18n("Alejandro Wainzinger"),
-            ki18n("Developer (xevix)"), "aikawarazuni@gmail.com", "http://awainzin-foss.blogspot.com" );
+    //Currently active Authors
+    extern OcsData ocsData;
     aboutData.addAuthor( ki18n("Bart 'Where are my toothpicks' Cerneels"),
             ki18n("Developer (Stecchino)"), "bart.cerneels@kde.org", "http://commonideas.blogspot.com" );
-    aboutData.addAuthor( ki18n("Dan 'Hey, it compiled...' Meltzer"),
-            ki18n("Developer (hydrogen)"), "parallelgrapefruit@gmail.com" );
-    aboutData.addAuthor( ki18n("Ian 'The Beard' Monroe"),
-            ki18n("Developer (eean)"), "ian@monroe.nu", "http://www.monroe.nu/" );
-    aboutData.addAuthor( ki18n("Jeff 'IROKSOHARD' Mitchell"),
-            ki18n("Developer (jefferai)"), "mitchell@kde.org", "http://amarok.kde.org/blog/categories/13-jefferai" );
-    aboutData.addAuthor( ki18n("Leo Franchi"),
-            ki18n("Developer (lfranchi)"), "lfranchi@kde.org" );
-    aboutData.addAuthor( ki18n("Lydia 'is wrong(TM)' Pintscher"),
-            ki18n("Release Vixen (Nightrose)"), "lydia@kde.org" );
-    aboutData.addAuthor( ki18n("Mark 'It's good, but it's not irssi' Kretschmann"), //krazy:exclude=contractions
-            ki18n("Project founder (markey)"), "kretschmann@kde.org" );
-    aboutData.addAuthor( ki18n("Maximilian Kossick"),
-            ki18n("Developer (maxx_k)"), "maximilian.kossick@gmail.com" );
-    aboutData.addAuthor( ki18n("Nikolaj Hald 'Also very hot' Nielsen"),
-            ki18n("Developer (nhn)"), "nhn@kde.org", "http://amarok.kde.org/blog/categories/18-freespirit" );
-    aboutData.addAuthor( ki18n("Seb 'Surfin' down under' Ruiz"),
-            ki18n("Developer (sebr)"), "ruiz@kde.org", "http://www.sebruiz.net" );
+    ocsData.addAuthor( "Stecchino", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Edward \"Hades\" Toroshchin"),
+            ki18n("Developer (dr_lepper)"), "edward.hades@gmail.com" );
+    ocsData.addAuthor( "hadeschief", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Mark Kretschmann" ),
+            ki18n("Project founder (markey)"), "kretschmann@kde.org", "https://plus.google.com/102602725322221030250/posts" );
+        ocsData.addAuthor( "MarkKretschmann", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Matěj Laitl"),
+            ki18n("iPod collection rewrite & more (strohel)"), "matej@laitl.cz", "http://strohel.blogspot.com/" );
+    ocsData.addAuthor( "strohel", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Myriam Schweingruber"), ki18n("Rokymoter, Bug triaging (Mamarok)"), "myriam@kde.org", "http://blogs.fsfe.org/myriam" );
+    ocsData.addAuthor( "Mamarok", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Ralf 'SalsaMaster' Engels"),
+            ki18n("Developer (rengels)"), "ralf.engels@nokia.com" );
+    ocsData.addAuthor( QString(), aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Patrick von Reth"), ki18n("Windows build (TheOneRing)"),
+            "patrick.vonreth@gmail.com" );
+    ocsData.addAuthor( QString(), aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Rick W. Chen"),
+            ki18n("Developer (stuffcorpse)"), "stuffcorpse@archlinux.us" );
+    ocsData.addAuthor( "stuffcorpse", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Sam Lade"), ki18n("Developer (Sentynel)"),
+            "sam@sentynel.com" );
+    ocsData.addAuthor( "Sentynel", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Sven Krohlas"), ki18n("Rokymoter, Developer (sven423)"), "sven@asbest-online.de" );
+    ocsData.addAuthor( "krohlas", aboutData.authors().last() );
+
     aboutData.addAuthor( ki18n("Téo Mrnjavac"),
-            ki18n("Developer (Teo`)"), "teo.mrnjavac@gmail.com", "http://teom.wordpress.com/" );
+            ki18n("Developer (Teo`)"), "teo@kde.org", "http://teom.wordpress.com/" );
+    ocsData.addAuthor( "teom", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Valorie Zimmerman"),
+            ki18n("Rokymoter, Handbook (valorie)"), "valorie@kde.org" );
+    ocsData.addAuthor( "valorie", aboutData.authors().last() );
+
+    //Inactive authors
+    /* This list should contain people who still hold major copyright on the current code
+     * For instance: does not include authors of 1.4 who have not contributed to 2.x */
+    aboutData.addAuthor( ki18n("<i>Inactive authors</i>"),
+                         ki18n("Amarok authorship is not a hobby, it's a lifestyle. "
+                               "But when people move on we want to keep respecting "
+                               "them by mentioning them here:"), "" );
+        ocsData.addAuthor( "%%category%%", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Ian 'The Beard' Monroe"), ki18n("Developer (eean)"), "ian@monroe.nu" );
+        ocsData.addAuthor( "eean", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Jeff 'IROKSOHARD' Mitchell"), ki18n("Developer (jefferai)"), "mitchell@kde.org" );
+        ocsData.addAuthor( "jefferai", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Leo Franchi"), ki18n("Developer (lfranchi)"), "lfranchi@kde.org" );
+        ocsData.addAuthor( "lfranchi", aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Lydia 'is wrong(TM)' Pintscher"), ki18n("Release Vixen (Nightrose)"), "lydia@kde.org" );
+        ocsData.addAuthor( "nightrose", aboutData.authors().last() );
+
+    aboutData.addCredit( ki18n("Max Howell"), ki18n("Developer, Vision"), "max.howell@methylblue.com" );
+        ocsData.addCredit( QString(), aboutData.credits().last() );
+
+    aboutData.addAuthor( ki18n("Maximilian Kossick"), ki18n("Developer (maxx_k)"), "maximilian.kossick@gmail.com" );
+        ocsData.addAuthor( QString(), aboutData.authors().last() );
+
+    aboutData.addAuthor( ki18n("Nikolaj Hald 'Also very hot' Nielsen"), ki18n("Developer (nhn)"), "nhn@kde.org" );
+        ocsData.addAuthor( "nhnFreespirit", aboutData.authors().last() );
+
+    aboutData.addCredit( ki18n("Seb 'Surfin' down under' Ruiz"), ki18n("Developer (sebr)"), "ruiz@kde.org" );
+           ocsData.addCredit( "seb", aboutData.credits().last() );
 
-    ocsData.addAuthor( "xevix", aboutData.authors().at( 0 ) );
-    ocsData.addAuthor( "Stecchino", aboutData.authors().at( 1 ) );
-    ocsData.addAuthor( QString(), aboutData.authors().at( 2 ) );
-    ocsData.addAuthor( "eean", aboutData.authors().at( 3 ) );
-    ocsData.addAuthor( "jefferai", aboutData.authors().at( 4 ) );
-    ocsData.addAuthor( "lfranchi", aboutData.authors().at( 5 ) );
-    ocsData.addAuthor( "nightrose", aboutData.authors().at( 6 ) );
-    ocsData.addAuthor( "MarkKretschmann", aboutData.authors().at( 7 ) );
-    ocsData.addAuthor( QString(), aboutData.authors().at( 8 ) );
-    ocsData.addAuthor( "nhnFreespirit", aboutData.authors().at( 9 ) );
-    ocsData.addAuthor( "seb", aboutData.authors().at( 10 ) );
-    ocsData.addAuthor( "teom", aboutData.authors().at( 11 ) );
 
     //Contributors
+    aboutData.addCredit( ki18n("Alejandro Wainzinger"), ki18n("Developer (xevix)"), "aikawarazuni@gmail.com" );
+        ocsData.addCredit( "xevix", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Alex Merry"), ki18n("Developer, Replay Gain support"), "kde@randomguy3.me.uk" );
         ocsData.addCredit( "randomguy3", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Casey Link"), ki18n("MP3tunes integration"), "unnamedrambler@gmail.com" );
@@ -81,20 +141,19 @@ int main( int argc, char *argv[] )
         ocsData.addCredit( "dangle", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Dan Leinir Turthra Jensen"), ki18n("Usability"), "admin@leinir.dk" );
         ocsData.addCredit( "leinir", aboutData.credits().last() );
+    aboutData.addCredit( ki18n("Dan 'Hey, it compiled...' Meltzer"), ki18n("Developer (hydrogen)"), "parallelgrapefruit@gmail.com" );
+        ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Daniel Caleb Jones"), ki18n("Biased playlists"), "danielcjones@gmail.com" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Daniel Dewald"), ki18n("Tag Guesser, Labels"), "Daniel.Dewald@time-shift.de" );
-        ocsData.addCredit( QString(), aboutData.credits().last() );
+    aboutData.addCredit( ki18n("Daniel Dewald"), ki18n("Tag Guesser, Labels, Spectrum Analyzer"), "Daniel.Dewald@time-shift.de" );
+        ocsData.addCredit( "TheCrasher", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Daniel Winter"), ki18n("Nepomuk integration"), "dw@danielwinter.de" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Edward \"Hades\" Toroshchin"), ki18n("Developer"), "edward.hades@gmail.com" );
-        ocsData.addCredit( "hadeschief", aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Frank Meerkötter"), ki18n("Podcast improvements"),
-                         "frank@meerkoetter.org" );
+      aboutData.addCredit( ki18n("Frank Meerkötter"), ki18n("Podcast improvements"), "frank@meerkoetter.org" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Greg Meyer"), ki18n("Live CD, Bug squashing (oggb4mp3)"), "greg@gkmweb.com" );
         ocsData.addCredit( "oggb4mp3", aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Harald Sitter"), ki18n("Rokymoter (apachelogger)"), "harald.sitter@kdemail.net" );
+    aboutData.addCredit( ki18n("Harald Sitter"), ki18n("Phonon, Lord-President of KDE Multimedia (apachelogger)"), "harald.sitter@kdemail.net" );
         ocsData.addCredit( "apachelogger", aboutData.credits().last() );
     aboutData.addCredit( ki18n("John Atkinson"), ki18n(( "Assorted patches" )), "john@fauxnetic.co.uk" );
         ocsData.addCredit( "fauxnetic", aboutData.credits().last() );
@@ -108,14 +167,14 @@ int main( int argc, char *argv[] )
         ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Ljubomir Simin"), ki18n("Rokymoter (ljubomir)"), "ljubomir.simin@gmail.com" );
         ocsData.addCredit( "ljubomir", aboutData.credits().last() );
+    aboutData.addCredit( ki18n("Lucas Gomes"), ki18n("Developer (MaskMaster)"), "x8lucas8x@gmail.com" );
+        ocsData.addCredit( "x8lucas8x", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Mathias Panzenböck"), ki18n("Podcast improvements"), "grosser.meister.morti@gmx.net" );
         ocsData.addCredit( "panzi", aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Max Howell"), ki18n("Developer, Vision"), "max.howell@methylblue.com" );
-        ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Mikko Caldara"), ki18n("Bug triaging and sanitizing"), "mikko.cal@gmail.com" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Myriam Schweingruber"), ki18n("Rokymoter, bug squashing (Mamarok)"), "myriam@kde.org" );
-        ocsData.addCredit( "Mamarok", aboutData.credits().last() );
+    aboutData.addCredit( ki18n("Nikhil Marathe"), ki18n("UPnP support and patches (nsm)"), "nsm.nikhil@gmail.com" );
+        ocsData.addCredit( "nikhilm", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Nuno Pinheiro"), ki18n("Artwork"), "nuno@oxygen-icons.org" );
         ocsData.addCredit( "nunopinheirokde", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Olivier Bédard"), ki18n("Website hosting"), "paleo@pwsp.net" );
@@ -124,18 +183,16 @@ int main( int argc, char *argv[] )
         ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Peter Zhou Lei"), ki18n("Scripting interface"), "peterzhoulei@gmail.com" );
         ocsData.addCredit( "peterzl", aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Rick W. Chen"), ki18n("Cover fetcher"), "stuffcorpse@archlinux.us" );
-        ocsData.addCredit( "stuffcorpse", aboutData.credits().last() );
+    aboutData.addCredit( ki18n("Phalgun Guduthur"), ki18n("Nepomuk Collection (phalgun)"), "me@phalgun.in" );
+        ocsData.addCredit( "phalgun", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Scott Wheeler"), ki18n("TagLib & ktrm code"), "wheeler@kde.org" );
         ocsData.addCredit( "wheels", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Shane King"), ki18n("Patches & Windows porting (shakes)"), "kde@dontletsstart.com" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Simon Esneault"), ki18n("Photos & Videos applets, Context View"), "simon.esneault@gmail.com" );
         ocsData.addCredit( "Takahani", aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Soren Harward"), ki18n("Developer"), "stharward@gmail.com" );
+    aboutData.addCredit( ki18n("Soren Harward"), ki18n("Developer, Automated Playlist Generator"), "stharward@gmail.com" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
-    aboutData.addCredit( ki18n("Sven Krohlas"), ki18n("Rokymoter, Developer (sven423)"), "sven@asbest-online.de" );
-        ocsData.addCredit( "krohlas", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Thomas Lübking"), ki18n("Developer"), "thomas.luebking@web.de" );
         ocsData.addCredit( "thomas12777", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Valentin Rouet"), ki18n("Developer"), "v.rouet@gmail.com" );
@@ -144,6 +201,8 @@ int main( int argc, char *argv[] )
         ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("William Viana Soares"), ki18n("Context view"), "vianasw@gmail.com" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
+
+    //Former Contributors
     aboutData.addCredit( ki18n("Former contributors"), ki18n("People listed below have contributed to Amarok in the past. Thank you!"), "" );
         ocsData.addCredit( "%%category%%", aboutData.credits().last() );
     aboutData.addCredit( ki18n("Adam Pigg"), ki18n("Analyzers, patches, shoutcast"), "adam@piggz.co.uk" );
@@ -204,9 +263,17 @@ int main( int argc, char *argv[] )
         ocsData.addCredit( QString(), aboutData.credits().last() );
     aboutData.addCredit( ki18n("Stefan Bogner"), ki18n("Loads of stuff"), "bochi@online.ms" );
         ocsData.addCredit( QString(), aboutData.credits().last() );
+    aboutData.addCredit( ki18n("Tomasz Dudzik"), ki18n("Splash screen"), "madsheytan@gmail.com" );
+        ocsData.addCredit( QString(), aboutData.credits().last() );
+
+    //Donors:
+    //Last update: 2012/11/07, post Roktober 2012
+    ocsData.addDonor( "ayleph", KAboutPerson( ki18n( "Andrew Browning" ) ) );
+    ocsData.addDonor( QString(), KAboutPerson( ki18n( "Chris Wales" ) ) );
+    ocsData.addDonor( QString(), KAboutPerson( ki18n( "ZImin Stanislav" ) ) );
 
     KCmdLineArgs::reset();
-    KCmdLineArgs::init( argc, argv, &::aboutData ); //calls KCmdLineArgs::addStdCmdLineOptions()
+    KCmdLineArgs::init( argc, argv, &aboutData ); //calls KCmdLineArgs::addStdCmdLineOptions()
 
     App::initCliArgs();
     KUniqueApplication::addCmdLineOptions();
@@ -214,7 +281,21 @@ int main( int argc, char *argv[] )
     KCmdLineArgs* const args = KCmdLineArgs::parsedArgs();
 
     KUniqueApplication::StartFlag startFlag;
-    startFlag = args->isSet( "multipleinstances" ) ? KUniqueApplication::NonUniqueInstance : KUniqueApplication::StartFlag( 0 );
+    startFlag = args->isSet( "multipleinstances" )
+                ? KUniqueApplication::NonUniqueInstance
+                : KUniqueApplication::StartFlag( 0 );
+
+    const bool debugColorsEnabled = !args->isSet( "coloroff" );
+    const bool debugEnabled = args->isSet( "debug" );
+
+    Debug::setDebugEnabled( debugEnabled );
+    Debug::setColoredDebug( debugColorsEnabled );
+
+    if ( args->isSet( "debug-audio" ) ) {
+        qputenv( "PHONON_DEBUG", QByteArray( "3" ) );
+        qputenv( "PHONON_BACKEND_DEBUG", QByteArray( "3" ) );
+        qputenv( "PHONON_PULSEAUDIO_DEBUG", QByteArray( "3" ) );
+    }
 
     if( !KUniqueApplication::start( startFlag ) ) {
         QList<QByteArray> instanceOptions;
@@ -232,6 +313,17 @@ int main( int argc, char *argv[] )
         return 0;
     }
 
+    // Rewrite default SIGINT and SIGTERM handlers
+    // to make amarok save current playlists during forced
+    // application termination (logout, Ctr+C in console etc.)
+    signal( SIGINT, &QCoreApplication::exit );
+    signal( SIGTERM, &QCoreApplication::exit );
+
+    // This call is needed to prevent a crash on exit with Phonon-VLC and LibPulse
+#ifdef Q_WS_X11
+    XInitThreads();
+#endif
+
     App app;
     app.setUniqueInstance( startFlag == KUniqueApplication::NonUniqueInstance );
     return app.exec();
