@@ -31,47 +31,60 @@
 #include "AmarokMimeData.h"
 #include "core/capabilities/ReadLabelCapability.h"
 #include "core/support/Debug.h"
-#include "DirectoryLoader.h"
 #include "EngineController.h"
+#include "core/capabilities/MultiSourceCapability.h"
 #include "core/capabilities/SourceInfoCapability.h"
 #include "core/collections/Collection.h"
+#include "core/meta/Statistics.h"
 #include "core/meta/support/MetaUtility.h"
-#include "PlaylistColumnNames.h"
+#include "PlaylistDefines.h"
 #include "PlaylistActions.h"
-#include "PlaylistModelStack.h"
+#include "PlaylistController.h"
 #include "PlaylistItem.h"
 #include "core-impl/playlists/types/file/PlaylistFileSupport.h"
-#include "UndoCommands.h"
+#include "core-impl/support/TrackLoader.h"
+#include "playlist/UndoCommands.h"
 
 #include <KGlobal>
-#include <KUrl>
 #include <KIconLoader>
+#include <KLocale>
+#include <KUrl>
 
 #include <QAction>
+#include <QTimer>
 #include <QDate>
 #include <QStringList>
 #include <QTextDocument>
 
 #include <typeinfo>
 
+#define TOOLTIP_STATIC_LINEBREAK 50
+
 bool Playlist::Model::s_tooltipColumns[NUM_COLUMNS];
+bool Playlist::Model::s_showToolTip;
 
 // ------- helper functions for the tooltip
 
+static bool
+fitsInOneLineHTML(const QString& text)
+{
+    // The size of the normal, standard line
+    const int lnSize = TOOLTIP_STATIC_LINEBREAK;
+    return (text.size() <= lnSize);
+}
+
 static QString
 breakLongLinesHTML(const QString& text)
 {
     // Now let's break up long lines so that the tooltip doesn't become hideously large
-
-    // The size of the normal, standard line
-    const int lnSize = 50;
-    if (text.size() <= lnSize)
+    if (fitsInOneLineHTML(text))
     {
         // If the text is not too long, return it as it is
         return text;
     }
     else
     {
+        const int lnSize = TOOLTIP_STATIC_LINEBREAK;
         QString textInLines;
 
         QStringList words = text.trimmed().split(' ');
@@ -119,7 +132,7 @@ HTMLLine( const Playlist::Column& column, const QString& value, bool force = fal
     {
         QString line;
         line += "<tr><td align=\"right\">";
-        line += "<img src=\""+KIconLoader::global()->iconPath( Playlist::iconNames[column] , -16)+"\" />";
+        line += "<img src=\""+KIconLoader::global()->iconPath( Playlist::iconName( column ), -16)+"\" />";
         line += "</td><td align=\"left\">";
         line += breakLongLinesHTML( value );
         line += "</td></tr>";
@@ -139,7 +152,9 @@ HTMLLine( const Playlist::Column& column, const QString& value, bool force = fal
 static QString
 HTMLLine( const Playlist::Column& column, const int value, bool force = false )
 {
-    if( (value != 0) || force )
+    // there is currenly no numeric meta-data that would have sense if it were negative.
+    // also, zero denotes not available, unknow etc; don't show these unless forced.
+    if( value > 0 || force )
     {
         return HTMLLine( column, QString::number( value ) );
     }
@@ -154,8 +169,24 @@ Playlist::Model::Model( QObject *parent )
         , m_totalLength( 0 )
         , m_totalSize( 0 )
         , m_setStateOfItem_batchMinRow( -1 )
+        , m_saveStateTimer( new QTimer(this) )
 {
     DEBUG_BLOCK
+
+    m_saveStateTimer->setInterval( 5000 );
+    m_saveStateTimer->setSingleShot( true );
+    connect( m_saveStateTimer, SIGNAL(timeout()),
+             this, SLOT(saveState()) );
+    connect( this, SIGNAL(modelReset()),
+             this, SLOT(queueSaveState()) );
+    connect( this, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+             this, SLOT(queueSaveState()) );
+    connect( this, SIGNAL(rowsInserted(QModelIndex,int,int)),
+             this, SLOT(queueSaveState()) );
+    connect( this, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)),
+             this, SLOT(queueSaveState()) );
+    connect( this, SIGNAL(rowsRemoved(QModelIndex,int,int)),
+             this, SLOT(queueSaveState()) );
 }
 
 Playlist::Model::~Model()
@@ -168,6 +199,32 @@ Playlist::Model::~Model()
     qDeleteAll( m_items );
 }
 
+void
+Playlist::Model::saveState()
+{
+    exportPlaylist( Amarok::defaultPlaylistPath() );
+}
+
+void
+Playlist::Model::queueSaveState()
+{
+    if ( !m_saveStateTimer->isActive() )
+        m_saveStateTimer->start();
+}
+
+void
+Playlist::Model::insertTracksFromTrackLoader( const Meta::TrackList &tracks )
+{
+    QObject *loader = sender();
+    if( !sender() )
+    {
+        warning() << __PRETTY_FUNCTION__ << "can only be connected to TrackLoader";
+        return;
+    }
+    int insertRow = loader->property( "beginRow" ).toInt();
+    Controller::instance()->insertTracks( insertRow, tracks );
+}
+
 QVariant
 Playlist::Model::headerData( int section, Qt::Orientation orientation, int role ) const
 {
@@ -176,7 +233,7 @@ Playlist::Model::headerData( int section, Qt::Orientation orientation, int role
     if ( role != Qt::DisplayRole )
         return QVariant();
 
-    return columnNames( section );
+    return columnName( static_cast<Playlist::Column>( section ) );
 }
 
 void
@@ -186,16 +243,28 @@ Playlist::Model::setTooltipColumns( bool columns[] )
         s_tooltipColumns[i] = columns[i];
 }
 
+void
+Playlist::Model::enableToolTip( bool enable )
+{
+    s_showToolTip = enable;
+}
+
 QString
 Playlist::Model::tooltipFor( Meta::TrackPtr track ) const
 {
     QString text;
+    KLocale *locale = KGlobal::locale();
     // get the shared pointers now to be thread safe
     Meta::ArtistPtr artist = track->artist();
     Meta::AlbumPtr album = track->album();
+    Meta::ArtistPtr albumArtist = album ? album->albumArtist() : Meta::ArtistPtr();
     Meta::GenrePtr genre = track->genre();
     Meta::ComposerPtr composer = track->composer();
     Meta::YearPtr year = track->year();
+    Meta::StatisticsPtr statistics = track->statistics();
+
+    if( !track->isPlayable() )
+        text += i18n( "<b>Note:</b> This track is not playable.<br>%1", track->notPlayableReason() );
 
     if( s_tooltipColumns[Playlist::Title] )
         text += HTMLLine( Playlist::Title, track->name() );
@@ -203,6 +272,10 @@ Playlist::Model::tooltipFor( Meta::TrackPtr track ) const
     if( s_tooltipColumns[Playlist::Artist] && artist )
         text += HTMLLine( Playlist::Artist, artist->name() );
 
+    // only show albumArtist when different from artist (it should suffice to compare pointers)
+    if( s_tooltipColumns[Playlist::AlbumArtist] && albumArtist && albumArtist != artist )
+        text += HTMLLine( Playlist::AlbumArtist, albumArtist->name() );
+
     if( s_tooltipColumns[Playlist::Album] && album )
         text += HTMLLine( Playlist::Album, album->name() );
 
@@ -221,17 +294,41 @@ Playlist::Model::tooltipFor( Meta::TrackPtr track ) const
     if( s_tooltipColumns[Playlist::Year] && year && year->year() > 0 )
         text += HTMLLine( Playlist::Year, year->year() );
 
-    if( s_tooltipColumns[Playlist::Comment])
-        text += HTMLLine( Playlist::Comment, track->comment() );
+    if( s_tooltipColumns[Playlist::Bpm] )
+        text += HTMLLine( Playlist::Bpm, track->bpm() );
+
+    if( s_tooltipColumns[Playlist::Comment]) {
+        if ( !(fitsInOneLineHTML( track->comment() ) ) )
+            text += HTMLLine( Playlist::Comment, i18n( "(...)" ) );
+        else
+            text += HTMLLine( Playlist::Comment, track->comment() );
+    }
+
+    if( s_tooltipColumns[Playlist::Labels] && !track->labels().empty() )
+    {
+        QStringList labels;
+        foreach( Meta::LabelPtr label, track->labels() )
+        {
+            if( label )
+                labels << label->name();
+        }
+        text += HTMLLine( Playlist::Labels, labels.join( QString(", ") ) );
+    }
 
     if( s_tooltipColumns[Playlist::Score] )
-        text += HTMLLine( Playlist::Score, track->score() );
+        text += HTMLLine( Playlist::Score, statistics->score() );
 
     if( s_tooltipColumns[Playlist::Rating] )
-        text += HTMLLine( Playlist::Rating, QString::number( static_cast<double>(track->rating())/2.0 ) );
+        text += HTMLLine( Playlist::Rating, QString::number( statistics->rating()/2.0 ) );
 
     if( s_tooltipColumns[Playlist::PlayCount] )
-        text += HTMLLine( Playlist::PlayCount, track->playCount(), true );
+        text += HTMLLine( Playlist::PlayCount, statistics->playCount(), true );
+
+    if( s_tooltipColumns[Playlist::LastPlayed] && statistics->lastPlayed().isValid() )
+        text += HTMLLine( Playlist::LastPlayed, locale->formatDateTime( statistics->lastPlayed() ) );
+
+    if( s_tooltipColumns[Playlist::Bitrate] && track->bitrate() )
+        text += HTMLLine( Playlist::Bitrate, i18nc( "%1: bitrate", "%1 kbps", track->bitrate() ) );
 
     if( text.isEmpty() )
         text = QString( i18n( "No extra information available" ) );
@@ -268,93 +365,108 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
         return  m_items.at( row )->track()->inCollection();
 
     else if ( role == MultiSourceRole )
-        return  m_items.at( row )->track()->hasCapabilityInterface( Capabilities::Capability::MultiSource );
+        return  m_items.at( row )->track()->has<Capabilities::MultiSourceCapability>();
 
     else if ( role == StopAfterTrackRole )
         return Actions::instance()->willStopAfterTrack( idAt( row ) );
 
     else if ( role == Qt::ToolTipRole )
-        return tooltipFor( m_items.at( row )->track() );
+    {
+        Meta::TrackPtr track = m_items.at( row )->track();
+        if( s_showToolTip )
+            return tooltipFor( track );
+        else if( !track->isPlayable() )
+            return i18n( "<b>Note:</b> This track is not playable.<br>%1", track->notPlayableReason() );
+    }
 
     else if ( role == Qt::DisplayRole )
     {
-        Meta::AlbumPtr album = m_items.at( row )->track()->album();
+        Meta::TrackPtr track = m_items.at( row )->track();
+        Meta::AlbumPtr album = track->album();
+        Meta::StatisticsPtr statistics = track->statistics();
         switch ( index.column() )
         {
             case PlaceHolder:
-                return QString();
+                break;
             case Album:
             {
                 if( album )
                     return album->name();
-                return QString();
+                break;
             }
             case AlbumArtist:
             {
-                if( album && album->albumArtist() )
-                    return album->albumArtist()->name();
-                return QString();
+                if( album )
+                {
+                    Meta::ArtistPtr artist = album->albumArtist();
+                    if( artist )
+                        return artist->name();
+                }
+                break;
             }
             case Artist:
             {
-                if ( m_items.at( row )->track()->artist() )
-                    return m_items.at( row )->track()->artist()->name();
-                return QString();
+                Meta::ArtistPtr artist = track->artist();
+                if( artist )
+                    return artist->name();
+                break;
             }
             case Bitrate:
             {
-                return Meta::prettyBitrate( m_items.at( row )->track()->bitrate() );
+                return Meta::prettyBitrate( track->bitrate() );
             }
             case Bpm:
             {
-                if ( m_items.at( row )->track()->bpm() > 0.0 )
-                    return QString::number( m_items.at( row )->track()->bpm() );
-                return QString();
+                if( track->bpm() > 0.0 )
+                    return QString::number( track->bpm() );
+                break;
             }
             case Comment:
             {
-                return m_items.at( row )->track()->comment();
+                return track->comment();
             }
             case Composer:
             {
-                if ( m_items.at( row )->track()->composer() )
-                    return m_items.at( row )->track()->composer()->name();
-                return QString();
+                Meta::ComposerPtr composer = track->composer();
+                if( composer )
+                    return composer->name();
+                break;
             }
             case CoverImage:
             {
                 if( album )
                     return The::svgHandler()->imageWithBorder( album, 100 ); //FIXME:size?
-                return QImage();
+                break;
             }
             case Directory:
             {
-                if ( m_items.at( row )->track()->playableUrl().isLocalFile() )
-                    return m_items.at( row )->track()->playableUrl().directory();
-                else
-                    return QString();
+                if( track->playableUrl().isLocalFile() )
+                    return track->playableUrl().directory();
+                break;
             }
             case DiscNumber:
             {
-                return m_items.at( row )->track()->discNumber();
+                if( track->discNumber() > 0 )
+                    return track->discNumber();
+                break;
             }
             case Filename:
             {
 
-                if ( m_items.at( row )->track()->playableUrl().isLocalFile() )
-                    return m_items.at( row )->track()->playableUrl().fileName();
-                else
-                    return QString();
+                if( track->playableUrl().isLocalFile() )
+                    return track->playableUrl().fileName();
+                break;
             }
             case Filesize:
             {
-                return Meta::prettyFilesize( m_items.at( row )->track()->filesize() );
+                return Meta::prettyFilesize( track->filesize() );
             }
             case Genre:
             {
-                if ( m_items.at( row )->track()->genre() )
-                    return m_items.at( row )->track()->genre()->name();
-                return QString();
+                Meta::GenrePtr genre = track->genre();
+                if( genre )
+                    return genre->name();
+                break;
             }
             case GroupLength:
             {
@@ -366,7 +478,6 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case Labels:
             {
-                Meta::TrackPtr track = m_items.at( row )->track();
                 if( track )
                 {
                     QStringList labelNames;
@@ -376,22 +487,22 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
                     }
                     return labelNames.join( ", " );
                 }
-                return QString();
+                break;
             }
             case LastPlayed:
             {
-                if( m_items.at( row )->track()->lastPlayed().isValid() )
-                    return Amarok::verboseTimeSince( m_items.at( row )->track()->lastPlayed() );
+                if( statistics->lastPlayed().isValid() )
+                    return Amarok::verboseTimeSince( statistics->lastPlayed() );
                 else
                     return i18nc( "The amount of time since last played", "Never" );
             }
             case Length:
             {
-                return Meta::msToPrettyTime( m_items.at( row )->track()->length() );
+                return Meta::msToPrettyTime( track->length() );
             }
             case LengthInSeconds:
             {
-                return m_items.at( row )->track()->length() / 1000;
+                return track->length() / 1000;
             }
             case Mood:
             {
@@ -399,24 +510,26 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case PlayCount:
             {
-                return m_items.at( row )->track()->playCount();
+                return statistics->playCount();
             }
             case Rating:
             {
-                return m_items.at( row )->track()->rating();
+                return statistics->rating();
             }
             case SampleRate:
             {
-                return m_items.at( row )->track()->sampleRate();
+                if( track->sampleRate() > 0 )
+                    return track->sampleRate();
+                break;
             }
             case Score:
             {
-                return (int)m_items.at( row )->track()->score(); // Cast to int, as we don't need to show the decimals in the view..
+                return int( statistics->score() ); // Cast to int, as we don't need to show the decimals in the view..
             }
             case Source:
             {
                 QString sourceName;
-                Capabilities::SourceInfoCapability *sic = m_items.at( row )->track()->create<Capabilities::SourceInfoCapability>();
+                Capabilities::SourceInfoCapability *sic = track->create<Capabilities::SourceInfoCapability>();
                 if ( sic )
                 {
                     sourceName = sic->sourceName();
@@ -424,14 +537,14 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
                 }
                 else
                 {
-                    sourceName = m_items.at( row )->track()->collection() ? m_items.at( row )->track()->collection()->prettyName() : QString();
+                    sourceName = track->collection() ? track->collection()->prettyName() : QString();
                 }
                 return sourceName;
             }
             case SourceEmblem:
             {
                 QPixmap emblem;
-                Capabilities::SourceInfoCapability *sic = m_items.at( row )->track()->create<Capabilities::SourceInfoCapability>();
+                Capabilities::SourceInfoCapability *sic = track->create<Capabilities::SourceInfoCapability>();
                 if ( sic )
                 {
                     QString source = sic->sourceName();
@@ -443,15 +556,15 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case Title:
             {
-                return m_items.at( row )->track()->prettyName();
+                return track->prettyName();
             }
             case TitleWithTrackNum:
             {
                 QString trackString;
-                QString trackName = m_items.at( row )->track()->prettyName();
-                if ( m_items.at( row )->track()->trackNumber() > 0 )
+                QString trackName = track->prettyName();
+                if( track->trackNumber() > 0 )
                 {
-                    QString trackNumber = QString::number( m_items.at( row )->track()->trackNumber() );
+                    QString trackNumber = QString::number( track->trackNumber() );
                     trackString =  QString( trackNumber + " - " + trackName );
                 } else
                     trackString = trackName;
@@ -460,20 +573,23 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
             }
             case TrackNumber:
             {
-                return m_items.at( row )->track()->trackNumber();
+                if( track->trackNumber() > 0 )
+                    return track->trackNumber();
+                break;
             }
             case Type:
             {
-                return m_items.at( row )->track()->type();
+                return track->type();
             }
             case Year:
             {
-                if ( m_items.at( row )->track()->year() )
-                    return m_items.at( row )->track()->year()->name();
-                return QString();
+                Meta::YearPtr year = track->year();
+                if( year && year->year() > 0 )
+                    return year->year();
+                break;
             }
             default:
-                return QString();
+                return QVariant(); // returning a variant instead of a string inside a variant is cheaper
 
         }
     }
@@ -518,9 +634,8 @@ Playlist::Model::mimeData( const QModelIndexList &indexes ) const
 }
 
 bool
-Playlist::Model::dropMimeData( const QMimeData* data, Qt::DropAction action, int row, int, const QModelIndex& parent )
+Playlist::Model::dropMimeData( const QMimeData* data, Qt::DropAction action, int row, int, const QModelIndex &parent )
 {
-    DEBUG_BLOCK
     if ( action == Qt::IgnoreAction )
         return true;
 
@@ -584,8 +699,9 @@ Playlist::Model::dropMimeData( const QMimeData* data, Qt::DropAction action, int
     else if( data->hasUrls() )
     {
         debug() << "this is _something_ with a url....";
-        DirectoryLoader* dl = new DirectoryLoader(); //this deletes itself
-        dl->insertAtRow( beginRow );
+        TrackLoader *dl = new TrackLoader(); // auto-deletes itself
+        dl->setProperty( "beginRow", beginRow );
+        connect( dl, SIGNAL(finished(Meta::TrackList)), SLOT(insertTracksFromTrackLoader(Meta::TrackList)) );
         dl->init( data->urls() );
         return true;
     }
@@ -632,7 +748,7 @@ Playlist::Model::stateOfRow( int row ) const
 }
 
 bool
-Playlist::Model::containsTrack( const Meta::TrackPtr track ) const
+Playlist::Model::containsTrack( const Meta::TrackPtr& track ) const
 {
     foreach( Item* i, m_items )
     {
@@ -643,7 +759,7 @@ Playlist::Model::containsTrack( const Meta::TrackPtr track ) const
 }
 
 int
-Playlist::Model::firstRowForTrack( const Meta::TrackPtr track ) const
+Playlist::Model::firstRowForTrack( const Meta::TrackPtr& track ) const
 {
     int row = 0;
     foreach( Item* i, m_items )
@@ -656,7 +772,7 @@ Playlist::Model::firstRowForTrack( const Meta::TrackPtr track ) const
 }
 
 QSet<int>
-Playlist::Model::allRowsForTrack( const Meta::TrackPtr track ) const
+Playlist::Model::allRowsForTrack( const Meta::TrackPtr& track ) const
 {
     QSet<int> trackRows;
 
@@ -735,8 +851,6 @@ Playlist::Model::stateOfId( quint64 id ) const
 void
 Playlist::Model::metadataChanged( Meta::TrackPtr track )
 {
-    DEBUG_BLOCK;
-
     int row = 0;
     foreach( Item* i, m_items )
     {
@@ -748,7 +862,6 @@ Playlist::Model::metadataChanged( Meta::TrackPtr track )
                 subscribeTo( album );
 
             emit dataChanged( index( row, 0 ), index( row, columnCount() - 1 ) );
-            debug()<<"Metadata updated for track"<<track->prettyName();
         }
         row++;
     }
@@ -779,7 +892,7 @@ Playlist::Model::metadataChanged( Meta::AlbumPtr album )
 }
 
 bool
-Playlist::Model::exportPlaylist( const QString &path ) const
+Playlist::Model::exportPlaylist( const QString &path, bool relative ) const
 {
     // check queue state
     QQueue<quint64> queueIds = The::playlistActions()->queue();
@@ -787,7 +900,7 @@ Playlist::Model::exportPlaylist( const QString &path ) const
     foreach( quint64 id, queueIds ) {
       queued << rowForId( id );
     }
-    return Playlists::exportPlaylistFile( tracks(), path, queued );
+    return Playlists::exportPlaylistFile( tracks(), path, relative, queued );
 }
 
 Meta::TrackList
@@ -799,74 +912,6 @@ Playlist::Model::tracks() const
     return tl;
 }
 
-QString
-Playlist::Model::generatePlaylistName() const
-{
-    QString datePart = KGlobal::locale()->formatDateTime( QDateTime::currentDateTime(),
-                                                          KLocale::ShortDate, true );
-    Meta::TrackList trackList = tracks();
-
-    if( trackList.count() == 0 )
-    {
-        return i18nc( "A saved playlist with the current time (KLocale::Shortdate) added between \
-                      the parentheses",
-                      "Empty Playlist (%1)").arg( datePart );
-    }
-
-    bool singleArtist = true;
-    bool singleAlbum = true;
-
-    Meta::ArtistPtr artist = trackList.first()->artist();
-    Meta::AlbumPtr album = trackList.first()->album();
-
-    QString artistPart;
-    QString albumPart;
-
-    foreach( const Meta::TrackPtr track, trackList )
-    {
-        if( artist != track->artist() )
-            singleArtist = false;
-
-        if( album != track->album() )
-            singleAlbum = false;
-
-        if ( !singleArtist && !singleAlbum )
-            break;
-    }
-
-    if( ( !singleArtist && !singleAlbum ) ||
-        ( !artist && !album ) )
-        return i18nc( "A saved playlist with the current time (KLocale::Shortdate) added between \
-                      the parentheses",
-                      "Various Tracks (%1)" ).arg( datePart );
-
-    if( singleArtist )
-    {
-        if( artist )
-            artistPart = artist->prettyName();
-        else
-            artistPart = i18n( "Unknown Artist(s)" );
-    }
-    else if( album && album->hasAlbumArtist() && singleAlbum )
-        artistPart = album->albumArtist()->prettyName();
-    else
-        artistPart = i18n( "Various Artists" );
-
-    if( singleAlbum )
-    {
-        if( album )
-            albumPart = album->prettyName();
-        else
-            albumPart = i18n( "Unknown Album(s)" );
-    }
-    else
-        albumPart = i18n( "Various Albums" );
-
-    return i18nc( "A saved playlist titled <artist> - <album>", "%1 - %2")
-            .arg( artistPart, albumPart );
-
-}
-
 QString
 Playlist::Model::prettyColumnName( Column index ) //static
 {
@@ -946,122 +991,127 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
         m_activeRow += activeShift;
     else
     {
-         // If one of the inserted tracks is currently playing, choose it as the active track.
-        const Meta::TrackPtr engineTrack = The::engineController()->currentTrack();
-        if( engineTrack )
+        EngineController *engine = The::engineController();
+        if( engine ) // test cases might create a playlist without having an EngineController
         {
-            int engineRow = firstRowForTrack( engineTrack );
-            if( engineRow > -1 )
-                setActiveRow( engineRow );
+            const Meta::TrackPtr engineTrack = engine->currentTrack();
+            if( engineTrack )
+            {
+                int engineRow = firstRowForTrack( engineTrack );
+                if( engineRow > -1 )
+                    setActiveRow( engineRow );
+            }
         }
     }
 }
 
+static bool
+removeCmdLessThanByRow( const Playlist::RemoveCmd &left, const Playlist::RemoveCmd &right )
+{
+    return left.second < right.second;
+}
 
 void
-Playlist::Model::removeTracksCommand( const RemoveCmdList& cmds )
+Playlist::Model::removeTracksCommand( const RemoveCmdList &passedCmds )
 {
     DEBUG_BLOCK
-    if ( cmds.size() < 1 )
+    if ( passedCmds.size() < 1 )
         return;
 
-    if ( cmds.size() == m_items.size() )
+    if ( passedCmds.size() == m_items.size() )
     {
         clearCommand();
         return;
     }
 
-    int activeShift = 0;
-    bool activeDeleted = false;
-    foreach( const RemoveCmd &rc, cmds )
+    // sort tracks to remove by their row
+    RemoveCmdList cmds( passedCmds );
+    qSort( cmds.begin(), cmds.end(), removeCmdLessThanByRow );
+
+    // update the active row
+    if( m_activeRow >= 0 )
     {
-        activeShift += ( rc.second < m_activeRow ) ? 1 : 0;
-        if ( rc.second == m_activeRow )
-            activeDeleted = true;
+        int activeShift = 0;
+        foreach( const RemoveCmd &rc, cmds )
+        {
+            if( rc.second < m_activeRow )
+                activeShift++;
+            else if( rc.second == m_activeRow )
+                m_activeRow = -1; // disappeared
+            else
+                break; // we got over it, nothing left to do
+        }
+        if( m_activeRow >= 0 ) // not deleted
+            m_activeRow -= activeShift;
     }
 
-    /* This next bit is probably more complicated that you expected it to be.
-     * The reason for the complexity comes from the following:
-     *
-     * 1. Qt's Model/View architecture can handle removal of only consecutive rows
-     * 2. The "remove rows" command from the Controller must handle
-     *    non-consecutive rows, and the removal command probably isn't sorted
-     *
-     * So each item has to be removed individually, and you can't just iterate
-     * over the commands, calling "m_items.removeAt(index)" as you go, because
-     * the indices of m_items will change with every removeAt().  Thus the
-     * following strategy of copying m_item, looking up the index in the copy and
-     * removing the item from the original list. (The strategy was changed from
-     * only replacing m_items at the end because that meant lying to the view -- Max)
-     *
-     * As a safety measure, the items themselves are not deleted until after m_items
-     * has been replaced.  If you delete as you go, then m_items will be holding
-     * dangling pointers, and the program will probably crash if the model is
-     * accessed in this state.   -- stharward */
+    QSet<Meta::TrackPtr> trackUnsubscribeCandidates;
+    QSet<Meta::AlbumPtr> albumUnsubscribeCandidates;
 
-    QList<Item*> originalList(m_items); // copy the current item list
-    QList<Item*> delitems;
-    foreach( const RemoveCmd &rc, cmds )
+    QListIterator<RemoveCmd> it( cmds );
+    int removedRows = 0;
+    while( it.hasNext() )
     {
-        Meta::TrackPtr track = rc.first;
-        Item* item = originalList.at(rc.second);
-        Q_ASSERT( track == item->track() );
+        int startRow = it.next().second;
+        int endRow = startRow;
 
-        // -- remove the items from the lists
-        int idx = rowForItem( item );
-        if (idx != -1) {
-            beginRemoveRows(QModelIndex(), idx, idx);
-            delitems.append(item);
-            m_items.removeAll( item );
-            m_itemIds.remove( item->id() );
-            endRemoveRows();
-        } else {
-            error() << "tried to delete a non-existent item:" << rc.first->prettyName() << rc.second;
+        // find consecutive runs of rows, this is important to group begin/endRemoveRows(),
+        // which are very costly when there are many proxymodels and a view above.
+        while( it.hasNext() && it.peekNext().second == endRow + 1 )
+        {
+            it.next();
+            endRow++;
         }
 
-        m_totalLength -= track->length();
-        m_totalSize -= track->filesize();
-
-        // -- unsubscribe
-        if( !containsTrack( track ) ) // check against same track two times in playlist
+        beginRemoveRows( QModelIndex(), startRow - removedRows, endRow - removedRows );
+        for( int row = startRow; row <= endRow; row++ )
         {
-            unsubscribeFrom( track );
+            Item *removedItem = m_items.at( row - removedRows );
+            m_items.removeAt( row - removedRows );
+            m_itemIds.remove( removedItem->id() );
 
+            const Meta::TrackPtr &track = removedItem->track();
+            // update totals here so they're right when endRemoveRows() called
+            m_totalLength -= track->length();
+            m_totalSize -= track->filesize();
+            trackUnsubscribeCandidates.insert( track );
             Meta::AlbumPtr album = track->album();
             if( album )
-            {
-                // check if we are the last track in playlist with this album
-                bool last = true;
-                const int size = m_items.size();
-                for ( int i = 0; i < size; i++ )
-                {
-                    if( m_items.at( i )->track()->album() == album )
-                    {
-                        last = false;
-                        break;
-                    }
-                }
-                if( last )
-                    unsubscribeFrom( album );
-            }
+                albumUnsubscribeCandidates.insert( album );
+
+            delete removedItem; // note track is by reference, needs removedItem alive
+            removedRows++;
         }
+        endRemoveRows();
     }
 
-    qDeleteAll(delitems);
-    delitems.clear();
-
-    //update the active row
-    if ( !activeDeleted && ( m_activeRow >= 0 ) )
+    // unsubscribe from tracks no longer present in playlist
+    foreach( Meta::TrackPtr track, trackUnsubscribeCandidates )
     {
-        m_activeRow = ( m_activeRow > 0 ) ? m_activeRow - activeShift : 0;
-    }
-    else
-    {
-        m_activeRow = -1;
+        if( !containsTrack( track ) )
+            unsubscribeFrom( track );
     }
 
-    //make sure that there are enough tracks if we just removed from a dynamic playlist.
-    Playlist::Actions::instance()->normalizeDynamicPlaylist();
+    // unsubscribe from albums no longer present im playlist
+    QSet<Meta::AlbumPtr> remainingAlbums;
+    foreach( const Item *item, m_items )
+    {
+        Meta::AlbumPtr album = item->track()->album();
+        if( album )
+            remainingAlbums.insert( album );
+    }
+    foreach( Meta::AlbumPtr album, albumUnsubscribeCandidates )
+    {
+        if( !remainingAlbums.contains( album ) )
+            unsubscribeFrom( album );
+    }
+
+    // make sure that there are enough tracks if we just removed from a dynamic playlist.
+    // This call needs to be delayed or else we would mess up the undo queue
+    // BUG: 259675
+    // FIXME: removing the track and normalizing the playlist should be grouped together
+    //        so that an undo operation undos both.
+    QTimer::singleShot(0, Playlist::Actions::instance(), SLOT(normalizeDynamicPlaylist()));
 }
 
 
@@ -1097,14 +1147,18 @@ Playlist::Model::moveTracksCommand( const MoveCmdList& cmds, bool reverse )
     if ( cmds.size() < 1 )
         return;
 
-    int min = m_items.size() + cmds.size();
-    int max = 0;
+    int min = INT_MAX;
+    int max = INT_MIN;
     foreach( const MoveCmd &rc, cmds )
     {
         min = qMin( min, rc.first );
-        min = qMin( min, rc.second );
         max = qMax( max, rc.first );
-        max = qMax( max, rc.second );
+    }
+
+    if( min < 0 || max >= m_items.size() )
+    {
+        error() << "Wrong row numbers given";
+        return;
     }
 
     int newActiveRow = m_activeRow;
@@ -1129,7 +1183,7 @@ Playlist::Model::moveTracksCommand( const MoveCmdList& cmds, bool reverse )
     }
 
     // We have 3 choices:
-    //   - Qt 4.6 'beginMoveRows()' / 'endMoveRows()'. Drawback: we'd need to do N of them, all causing resorts etc.
+    //   - Call 'beginMoveRows()' / 'endMoveRows()'. Drawback: we'd need to do N of them, all causing resorts etc.
     //   - Emit 'layoutAboutToChange' / 'layoutChanged'. Drawback: unspecific, 'changePersistentIndex()' complications.
     //   - Emit 'dataChanged'. Drawback: a bit inappropriate. But not wrong.
     emit dataChanged( index( min, 0 ), index( max, columnCount() - 1 ) );
