@@ -17,12 +17,11 @@
 #ifndef PROGRESSIVESEARCHWIDGET_H
 #define PROGRESSIVESEARCHWIDGET_H
 
+#include "widgets/LineEdit.h"
+
 #include <KVBox>
 
-#include "LineEdit.h"
-
 class KAction;
-
 class QKeyEvent;
 class QLabel;
 class QMenu;
@@ -58,12 +57,12 @@ public:
      */
     ~ProgressiveSearchWidget();
 
-    QString currentFilter() { return m_searchEdit->text(); }
+    QString currentFilter() const { return m_searchEdit->text(); }
 
     void focusInputLine();
     void setCurrentFilter( const QString filterExpr ) { m_searchEdit->setText( filterExpr ); }
 
-    bool onlyMatches() { return m_showOnlyMatches; }
+    bool onlyMatches() const { return m_showOnlyMatches; }
 
 signals:
     /**
@@ -212,14 +211,10 @@ private:
      */
     void readConfig();
 
-    void hideHiddenTracksWarning();
-    void showHiddenTracksWarning();
-
     Amarok::LineEdit *m_searchEdit;
     KAction   *m_nextAction;
     KAction   *m_previousAction;
     QMenu     *m_menu;
-    QLabel    *m_warningLabel;
     QToolBar  *m_toolBar;
 
     int        m_searchFieldsMask;
