@@ -20,22 +20,22 @@
 
 #include "DynamicTrackNavigator.h"
 
+#include "amarokconfig.h"
+#include "core/meta/Meta.h"
 #include "core/support/Debug.h"
 #include "dynamic/DynamicPlaylist.h"
 #include "dynamic/DynamicModel.h"
-#include "DynamicPlaylist.h"
-#include "core/meta/Meta.h"
-#include "amarokconfig.h"
+#include "dynamic/DynamicPlaylist.h"
 #include "playlist/PlaylistController.h"
 
 Playlist::DynamicTrackNavigator::DynamicTrackNavigator()
     : m_playlist( 0 )
 {
-    connect( m_model->qaim(), SIGNAL( activeTrackChanged( quint64 ) ), SLOT( trackChanged() ) );
-    connect( m_model->qaim(), SIGNAL( modelReset() ), SLOT( repopulate() ) );
+    connect( m_model->qaim(), SIGNAL(activeTrackChanged(quint64)), SLOT(trackChanged()) );
+    connect( m_model->qaim(), SIGNAL(modelReset()), SLOT(repopulate()) );
 
-    connect( Dynamic::DynamicModel::instance(), SIGNAL( activeChanged( int ) ),
-             SLOT( activePlaylistChanged() ) );
+    connect( Dynamic::DynamicModel::instance(), SIGNAL(activeChanged(int)),
+             SLOT(activePlaylistChanged()) );
     activePlaylistChanged();
 }
 
@@ -48,9 +48,7 @@ Playlist::DynamicTrackNavigator::~DynamicTrackNavigator()
 void
 Playlist::DynamicTrackNavigator::receiveTracks( Meta::TrackList tracks )
 {
-    DEBUG_BLOCK
-
-    The::playlistController()->insertOptioned( tracks, Append );
+    The::playlistController()->insertOptioned( tracks );
 }
 
 void
@@ -87,8 +85,8 @@ Playlist::DynamicTrackNavigator::activePlaylistChanged()
 
     if( m_playlist )
     {
-        disconnect( m_playlist, SIGNAL( tracksReady( Meta::TrackList ) ),
-                    this, SLOT( receiveTracks( Meta::TrackList ) ) );
+        disconnect( m_playlist, SIGNAL(tracksReady(Meta::TrackList)),
+                    this, SLOT(receiveTracks(Meta::TrackList)) );
         m_playlist->requestAbort();
     }
 
@@ -99,8 +97,8 @@ Playlist::DynamicTrackNavigator::activePlaylistChanged()
     }
     else
     {
-        connect( m_playlist, SIGNAL( tracksReady( Meta::TrackList ) ),
-                 this, SLOT( receiveTracks( Meta::TrackList ) ) );
+        connect( m_playlist, SIGNAL(tracksReady(Meta::TrackList)),
+                 this, SLOT(receiveTracks(Meta::TrackList)) );
     }
 }
 
