@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2008-2011 Soren Harward <stharward@gmail.com>                          *
+ * Copyright (c) 2008-2012 Soren Harward <stharward@gmail.com>                          *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -17,11 +17,13 @@
 #ifndef APG_CONSTRAINTSOLVER
 #define APG_CONSTRAINTSOLVER
 
-#include "core/meta/Meta.h"
+#include "core/meta/forward_declarations.h"
 
+#include <QHash>
 #include <QMutex>
 #include <QString>
-#include <threadweaver/Job.h>
+
+#include <ThreadWeaver/Job>
 
 class ConstraintNode;
 
@@ -34,8 +36,8 @@ namespace APG {
         Q_OBJECT
 
         public:
-            typedef QHash< Meta::TrackList*, double > Population;
-            
+            typedef QHash<Meta::TrackList*, double> Population;
+
             static const int QUALITY_RANGE; // allows used to adjust speed/quality tradeoff
 
             ConstraintSolver( ConstraintNode*, int );
@@ -106,9 +108,7 @@ namespace APG {
             // internal mathematical parameters
             quint32 m_maxGenerations;
             quint32 m_populationSize;
-            quint32 m_minPlaylistSize;
             quint32 m_suggestedPlaylistSize;
-            quint32 m_maxPlaylistSize;
     };
 } // namespace APG
 
