@@ -21,7 +21,7 @@
 
 #include "amarok_export.h"
 #include "core/collections/Collection.h"
-#include "support/MemoryCollection.h"
+#include "core-impl/collections/support/MemoryCollection.h"
 #include "ServiceBase.h"
 
 #include <KIcon>
@@ -102,8 +102,6 @@ class AMAROK_EXPORT ServiceCollection : public Collections::Collection
         ServiceBase * m_service;
         QSharedPointer<MemoryCollection> m_mc;
 
-        ServiceMetaFactory * m_metaFactory;
-
         QString m_collectionId;
         QString m_prettyName;
 
