@@ -14,16 +14,15 @@
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
- 
-#include "ServiceSqlQueryMaker.h"
 
 #define DEBUG_PREFIX "ServiceSqlQueryMaker"
 
-#include "Debug.h"
+#include "ServiceSqlQueryMaker.h"
 
-#include "collection/CollectionManager.h"
-#include "collection/SqlStorage.h"
-#include "MetaConstants.h"
+#include "core/collections/support/SqlStorage.h"
+#include "core/meta/support/MetaConstants.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/support/CollectionManager.h"
 #include "ServiceSqlCollection.h"
 
 #include <threadweaver/Job.h>
@@ -31,7 +30,7 @@
 
 #include <QStack>
 
-using namespace Meta;
+using namespace Collections;
 
 class ServiceSqlWorkerThread : public ThreadWeaver::Job
 {
@@ -66,8 +65,8 @@ class ServiceSqlWorkerThread : public ThreadWeaver::Job
 
 struct ServiceSqlQueryMaker::Private
 {
-    enum QueryType { NONE, TRACK, ARTIST, ALBUM, GENRE, COMPOSER, YEAR, CUSTOM };
-    enum {TRACKS_TABLE = 1, ALBUMS_TABLE = 2, ARTISTS_TABLE = 4, GENRE_TABLE = 8};
+    enum QueryType { NONE, TRACK, ARTIST, ALBUM, ALBUMARTIST, GENRE, COMPOSER, YEAR, CUSTOM };
+    enum {TRACKS_TABLE = 1, ALBUMS_TABLE = 2, ARTISTS_TABLE = 4, GENRE_TABLE = 8, ALBUMARTISTS_TABLE = 16 };
     int linkedTables;
     QueryType queryType;
     QString query;
@@ -79,7 +78,6 @@ struct ServiceSqlQueryMaker::Private
     //bool includedBuilder;
     //bool collectionRestriction;
     AlbumQueryMode albumMode;
-    bool returnDataPtrs;
     bool withoutDuplicates;
     int maxResultSize;
     ServiceSqlWorkerThread *worker;
@@ -96,35 +94,20 @@ ServiceSqlQueryMaker::ServiceSqlQueryMaker( ServiceSqlCollection* collection, Se
     //d->includedBuilder = true;
     //d->collectionRestriction = false;
     d->worker = 0;
-    reset();
+
+    d->queryType = Private::NONE;
+    d->linkedTables = 0;
+    d->withoutDuplicates = false;
+    d->maxResultSize = -1;
+    d->andStack.push( true );
 }
 
 ServiceSqlQueryMaker::~ServiceSqlQueryMaker()
 {
+    // what about d->worker?
     delete d;
 }
 
-QueryMaker*
-ServiceSqlQueryMaker::reset()
-{
-    d->query.clear();
-    d->queryType = Private::NONE;
-    d->queryReturnValues.clear();
-    d->queryFrom.clear();
-    d->queryMatch.clear();
-    d->queryFilter.clear();
-    d->queryOrderBy.clear();
-    d->linkedTables = 0;
-    if( d->worker && d->worker->isFinished() )
-        delete d->worker;   //TODO error handling
-    d->returnDataPtrs = false;
-    d->withoutDuplicates = false;
-    d->maxResultSize = -1;
-    d->andStack.clear();
-    d->andStack.push( true );
-    return this;
-}
-
 void
 ServiceSqlQueryMaker::abortQuery()
 {
@@ -132,13 +115,6 @@ ServiceSqlQueryMaker::abortQuery()
         d->worker->requestAbort();
 }
 
-QueryMaker*
-ServiceSqlQueryMaker::setReturnResultAsDataPtrs( bool returnDataPtrs )
-{
-    d->returnDataPtrs = returnDataPtrs;
-    return this;
-}
-
 void
 ServiceSqlQueryMaker::run()
 {
@@ -153,7 +129,7 @@ ServiceSqlQueryMaker::run()
     else
     {
         d->worker = new ServiceSqlWorkerThread( this );
-        connect( d->worker, SIGNAL( done( ThreadWeaver::Job* ) ), SLOT( done( ThreadWeaver::Job* ) ) );
+        connect( d->worker, SIGNAL(done(ThreadWeaver::Job*)), SLOT(done(ThreadWeaver::Job*)) );
         ThreadWeaver::Weaver::instance()->enqueue( d->worker );
     }
 }
@@ -214,6 +190,21 @@ ServiceSqlQueryMaker::setQueryType( QueryType type)
         }
         return this;
 
+    case QueryMaker::AlbumArtist:
+        if( d->queryType == Private::NONE )
+        {
+            QString prefix = m_metaFactory->tablePrefix();
+            d->queryFrom = ' ' + prefix + "_tracks";
+            d->linkedTables |= Private::ALBUMARTISTS_TABLE;
+            d->queryType = Private::ALBUMARTIST;
+            d->withoutDuplicates = true;
+            d->queryReturnValues = QString( "albumartists.id, " ) +
+                                            "albumartists.name, " +
+                                            "albumartists.description ";
+            d->queryOrderBy += " GROUP BY " + prefix + "_tracks.id"; //fixes the same track being shown several times due to being in several genres
+        }
+        return this;
+
     case QueryMaker::Album:
         if( d->queryType == Private::NONE )
         {
@@ -269,6 +260,7 @@ ServiceSqlQueryMaker::setQueryType( QueryType type)
         d->queryType = Private::CUSTOM;*/
         return this;
     
+    case QueryMaker::Label:
     case QueryMaker::None:
         return this;
     }
@@ -277,31 +269,7 @@ ServiceSqlQueryMaker::setQueryType( QueryType type)
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::includeCollection( const QString &collectionId )
-{
-    Q_UNUSED( collectionId );
-  /*  if( !d->collectionRestriction )
-    {
-        d->includedBuilder = false;
-        d->collectionRestriction = true;
-    }
-    if( m_collection->collectionId() == collectionId )
-        d->includedBuilder = true;*/
-    return this;
-}
-
-QueryMaker*
-ServiceSqlQueryMaker::excludeCollection( const QString &collectionId )
-{
-     Q_UNUSED( collectionId );
-    /*d->collectionRestriction = true;
-    if( m_collection->collectionId() == collectionId )
-        d->includedBuilder = false;*/
-    return this;
-}
-
-QueryMaker*
-ServiceSqlQueryMaker::addMatch( const TrackPtr &track )
+ServiceSqlQueryMaker::addMatch( const Meta::TrackPtr &track )
 {
     //DEBUG_BLOCK
     Q_UNUSED( track );
@@ -310,29 +278,54 @@ ServiceSqlQueryMaker::addMatch( const TrackPtr &track )
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::addMatch( const ArtistPtr &artist )
+ServiceSqlQueryMaker::addMatch( const Meta::ArtistPtr &artist, QueryMaker::ArtistMatchBehaviour behaviour )
 {
     QString prefix = m_metaFactory->tablePrefix();
 
     if( !d )
         return this;
 
+    if( behaviour == AlbumArtists || behaviour == AlbumOrTrackArtists )
+        d->linkedTables |= Private::ALBUMARTISTS_TABLE;
+
     //this should NOT be made into a static cast as this might get called with an incompatible type!
-    const ServiceArtist * serviceArtist = dynamic_cast<const ServiceArtist *>( artist.data() );
+    const Meta::ServiceArtist * serviceArtist = dynamic_cast<const Meta::ServiceArtist *>( artist.data() );
     d->linkedTables |= Private::ARTISTS_TABLE;
     if( serviceArtist )
     {
-        d->queryMatch += QString( " AND " + prefix + "_artists.id= '%1'" ).arg( serviceArtist->id() );
+        switch( behaviour )
+        {
+            case TrackArtists:
+                 d->queryMatch += QString( " AND " + prefix + "_artists.id= '%1'" ).arg( serviceArtist->id() );
+                 break;
+            case AlbumArtists:
+                 d->queryMatch += QString( " AND albumartists.id= '%1'" ).arg( serviceArtist->id() );
+                 break;
+            case AlbumOrTrackArtists:
+                 d->queryMatch += QString( " AND ( " + prefix + "_artists.id= '%1' OR albumartists.id= '%1' )" ).arg( serviceArtist->id() );
+                 break;
+        }
     }
     else
     {
-        d->queryMatch += QString( " AND " + prefix + "_artists.name='%1'" ).arg( escape( artist->name() ) );
+        switch( behaviour )
+        {
+            case TrackArtists:
+                 d->queryMatch += QString( " AND " + prefix + "_artists.name= '%1'" ).arg( escape( artist->name() ) );
+                 break;
+            case AlbumArtists:
+                 d->queryMatch += QString( " AND albumartists.name= '%1'" ).arg( escape( artist->name() ) );
+                 break;
+            case AlbumOrTrackArtists:
+                 d->queryMatch += QString( " AND ( " + prefix + "_artists.name= '%1' OR albumartists.name= '%1' )" ).arg( escape( artist->name() ) );
+                 break;
+        }
     }
     return this;
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::addMatch( const AlbumPtr &album )
+ServiceSqlQueryMaker::addMatch( const Meta::AlbumPtr &album )
 {
     QString prefix = m_metaFactory->tablePrefix();
 
@@ -340,7 +333,7 @@ ServiceSqlQueryMaker::addMatch( const AlbumPtr &album )
         return this;
 
     //this should NOT be made into a static cast as this might get called with an incompatible type!
-    const ServiceAlbumPtr serviceAlbum = ServiceAlbumPtr::dynamicCast( album );
+    const Meta::ServiceAlbumPtr serviceAlbum = Meta::ServiceAlbumPtr::dynamicCast( album );
 
     d->linkedTables |= Private::ALBUMS_TABLE;
     d->linkedTables |= Private::ARTISTS_TABLE;
@@ -358,12 +351,12 @@ ServiceSqlQueryMaker::addMatch( const AlbumPtr &album )
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::addMatch( const GenrePtr &genre )
+ServiceSqlQueryMaker::addMatch( const Meta::GenrePtr &genre )
 {
     QString prefix = m_metaFactory->tablePrefix();
 
     //this should NOT be made into a static cast as this might get called with an incompatible type!
-    const ServiceGenre* serviceGenre = static_cast<const ServiceGenre *>( genre.data() );
+    const Meta::ServiceGenre* serviceGenre = static_cast<const Meta::ServiceGenre *>( genre.data() );
     if( !d || !serviceGenre )
         return this;
 
@@ -384,7 +377,7 @@ ServiceSqlQueryMaker::addMatch( const GenrePtr &genre )
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::addMatch( const ComposerPtr &composer )
+ServiceSqlQueryMaker::addMatch( const Meta::ComposerPtr &composer )
 {
     Q_UNUSED( composer );
     //TODO
@@ -392,7 +385,7 @@ ServiceSqlQueryMaker::addMatch( const ComposerPtr &composer )
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::addMatch( const YearPtr &year )
+ServiceSqlQueryMaker::addMatch( const Meta::YearPtr &year )
 {
     Q_UNUSED( year );
     //TODO
@@ -400,10 +393,10 @@ ServiceSqlQueryMaker::addMatch( const YearPtr &year )
 }
 
 QueryMaker*
-ServiceSqlQueryMaker::addMatch( const DataPtr &data )
+ServiceSqlQueryMaker::addMatch( const Meta::LabelPtr &label )
 {
-    ( const_cast<DataPtr&>(data) )->addMatchTo( this );
-    //TODO needed at all?
+    Q_UNUSED( label );
+    //TODO
     return this;
 }
 
@@ -491,13 +484,6 @@ ServiceSqlQueryMaker::orderBy( qint64 value, bool descending )
     return this;
 }
 
-QueryMaker*
-ServiceSqlQueryMaker::orderByRandom()
-{
-    // TODO
-    return this;
-}
-
 QueryMaker*
 ServiceSqlQueryMaker::limitMaxResultSize( int size )
 {
@@ -519,6 +505,8 @@ ServiceSqlQueryMaker::linkTables()
        d->queryFrom += " LEFT JOIN " + prefix + "_albums ON " + prefix + "_tracks.album_id = " + prefix + "_albums.id";
     if( d->linkedTables & Private::ARTISTS_TABLE )
        d->queryFrom += " LEFT JOIN " + prefix + "_artists ON " + prefix + "_albums.artist_id = " + prefix + "_artists.id";
+    if( d->linkedTables & Private::ALBUMARTISTS_TABLE )
+        d->queryFrom += " LEFT JOIN " + prefix + "_artists AS albumartists ON " + prefix + "_albums.artist_id = albumartists.id";
     if( d->linkedTables & Private::GENRE_TABLE )
        d->queryFrom += " LEFT JOIN " + prefix + "_genre ON " + prefix + "_genre.album_id = " + prefix + "_albums.id";
 }
@@ -577,12 +565,13 @@ ServiceSqlQueryMaker::handleResult( const QStringList &result )
     {
         switch( d->queryType ) {
         /*case Private::CUSTOM:
-            emit newResultReady( m_collection->collectionId(), result );
+            emit newResultReady( result );
             break;*/
         case Private::TRACK:
             handleTracks( result );
             break;
         case Private::ARTIST:
+        case Private::ALBUMARTIST:
             handleArtists( result );
             break;
         case Private::ALBUM:
@@ -604,33 +593,32 @@ ServiceSqlQueryMaker::handleResult( const QStringList &result )
             break;
         }
     }
-    else if( d->returnDataPtrs )
-    {
-        emit newResultReady( m_collection->collectionId(), Meta::DataList() );
-    }
     else
     {
         switch( d->queryType ) {
             case QueryMaker::Custom:
-                emit newResultReady( m_collection->collectionId(), QStringList() );
+                emit newResultReady( QStringList() );
                 break;
             case QueryMaker::Track:
-                emit newResultReady( m_collection->collectionId(), Meta::TrackList() );
+                emit newResultReady( Meta::TrackList() );
                 break;
             case QueryMaker::Artist:
-                emit newResultReady( m_collection->collectionId(), Meta::ArtistList() );
+                emit newResultReady( Meta::ArtistList() );
                 break;
             case QueryMaker::Album:
-                emit newResultReady( m_collection->collectionId(), Meta::AlbumList() );
+                emit newResultReady( Meta::AlbumList() );
+                break;
+            case QueryMaker::AlbumArtist:
+                emit newResultReady( Meta::ArtistList() );
                 break;
             case QueryMaker::Genre:
-                emit newResultReady( m_collection->collectionId(), Meta::GenreList() );
+                emit newResultReady( Meta::GenreList() );
                 break;
             case QueryMaker::Composer:
-                emit newResultReady( m_collection->collectionId(), Meta::ComposerList() );
+                emit newResultReady( Meta::ComposerList() );
                 break;
             case QueryMaker::Year:
-                emit newResultReady( m_collection->collectionId(), Meta::YearList() );
+                emit newResultReady( Meta::YearList() );
                 break;
 
         case QueryMaker::None:
@@ -677,24 +665,10 @@ ServiceSqlQueryMaker::nameForValue( qint64 value )
     }
 }
 
-template<class PointerType, class ListType>
-void ServiceSqlQueryMaker::emitProperResult( const ListType& list )
-{
-    if ( d->returnDataPtrs ) {
-        DataList data;
-        foreach( PointerType p, list )
-            data << DataPtr::staticCast( p );
-
-        emit newResultReady( m_collection->collectionId(), data );
-    }
-    else
-        emit newResultReady( m_collection->collectionId(), list );
-}
-
 void
 ServiceSqlQueryMaker::handleTracks( const QStringList &result )
 {
-    TrackList tracks;
+    Meta::TrackList tracks;
     //SqlRegistry* reg = m_collection->registry();
     int rowCount = ( m_metaFactory->getTrackSqlRowCount() +
                    m_metaFactory->getAlbumSqlRowCount() +
@@ -707,17 +681,17 @@ ServiceSqlQueryMaker::handleTracks( const QStringList &result )
     {
         QStringList row = result.mid( i*rowCount, rowCount );
 
-        TrackPtr trackptr =  m_registry->getTrack( row );
+        Meta::TrackPtr trackptr =  m_registry->getTrack( row );
         tracks.append( trackptr );
     }
 
-    emitProperResult<TrackPtr, TrackList>( tracks );
+    emit newResultReady( tracks );
 }
 
 void
 ServiceSqlQueryMaker::handleArtists( const QStringList &result )
 {
-    ArtistList artists;
+    Meta::ArtistList artists;
    // SqlRegistry* reg = m_collection->registry();
     int rowCount = m_metaFactory->getArtistSqlRowCount();
     int resultRows = result.size() / rowCount;
@@ -726,13 +700,13 @@ ServiceSqlQueryMaker::handleArtists( const QStringList &result )
         QStringList row = result.mid( i*rowCount, rowCount );
         artists.append( m_registry->getArtist( row ) );
     }
-    emitProperResult<ArtistPtr, ArtistList>( artists );
+    emit newResultReady( artists );
 }
 
 void
 ServiceSqlQueryMaker::handleAlbums( const QStringList &result )
 {
-    AlbumList albums;
+    Meta::AlbumList albums;
     int rowCount = m_metaFactory->getAlbumSqlRowCount() +  m_metaFactory->getArtistSqlRowCount();
     int resultRows = result.size() / rowCount;
 
@@ -741,13 +715,13 @@ ServiceSqlQueryMaker::handleAlbums( const QStringList &result )
         QStringList row = result.mid( i*rowCount, rowCount );
         albums.append( m_registry->getAlbum( row ) );
     }
-    emitProperResult<AlbumPtr, AlbumList>( albums );
+    emit newResultReady( albums );
 }
 
 void
 ServiceSqlQueryMaker::handleGenres( const QStringList &result )
 {
-    GenreList genres;
+    Meta::GenreList genres;
 
     int rowCount = m_metaFactory->getGenreSqlRowCount();
     int resultRows = result.size() / rowCount;
@@ -756,13 +730,13 @@ ServiceSqlQueryMaker::handleGenres( const QStringList &result )
         QStringList row = result.mid( i*rowCount, rowCount );
         genres.append( m_registry->getGenre( row ) );
     }
-    emitProperResult<GenrePtr, GenreList>( genres );
+    emit newResultReady( genres );
 }
 
 /*void
 ServiceSqlQueryMaker::handleComposers( const QStringList &result )
 {
-    ComposerList composers;
+    Meta::ComposerList composers;
     SqlRegistry* reg = m_collection->registry();
     for( QStringListIterator iter( result ); iter.hasNext(); )
     {
@@ -770,13 +744,13 @@ ServiceSqlQueryMaker::handleComposers( const QStringList &result )
         QString id = iter.next();
         composers.append( reg->getComposer( name, id.toInt() ) );
     }
-    emitProperResult<ComposerPtr, ComposerList>( composers );
+    emit newResultReady( composers );
 }
 
 void
 ServiceSqlQueryMaker::handleYears( const QStringList &result )
 {
-    YearList years;
+    Meta::YearList years;
     SqlRegistry* reg = m_collection->registry();
     for( QStringListIterator iter( result ); iter.hasNext(); )
     {
@@ -784,7 +758,7 @@ ServiceSqlQueryMaker::handleYears( const QStringList &result )
         QString id = iter.next();
         years.append( reg->getYear( name, id.toInt() ) );
     }
-    emitProperResult<YearPtr, YearList>( years );
+    emit newResultReady( years );
 }*/
 
 QString
@@ -870,6 +844,4 @@ ServiceSqlQueryMaker::setAlbumQueryMode(AlbumQueryMode mode)
     return this;
 }
 
-
 #include "ServiceSqlQueryMaker.moc"
-
