@@ -23,8 +23,17 @@
 #include "AmpacheService.h"
 #include "NetworkAccessManagerProxy.h"
 
+#include <KUrl>
+
 namespace Collections {
 
+/** QueryMaker for Ampache
+    Since Ampache only supports a very limited set of searches
+    this query maker is very restricted.
+    E.g. it does not support searches with AND and OR.
+    TODO: support tags
+    TODO: think about only using the MemoryQueryMaker
+*/
 class AmpacheServiceQueryMaker : public DynamicServiceQueryMaker
 {
     Q_OBJECT
@@ -39,10 +48,9 @@ public:
     virtual QueryMaker* setQueryType( QueryType type );
 
     using DynamicServiceQueryMaker::addMatch;
-    virtual QueryMaker* addMatch ( const Meta::ArtistPtr &artist );
-    virtual QueryMaker* addMatch ( const Meta::AlbumPtr &album );
-
-    virtual QueryMaker* setReturnResultAsDataPtrs ( bool resultAsDataPtrs );
+    virtual QueryMaker* addMatch( const Meta::TrackPtr &track );
+    virtual QueryMaker* addMatch( const Meta::ArtistPtr &artist, ArtistMatchBehaviour behaviour = TrackArtists );
+    virtual QueryMaker* addMatch( const Meta::AlbumPtr &album );
 
     virtual QueryMaker* addFilter( qint64 value, const QString &filter, bool matchBegin = false, bool matchEnd = false );
     virtual QueryMaker* addNumberFilter( qint64 value, qint64 filter, QueryMaker::NumberComparison compare );
@@ -50,32 +58,14 @@ public:
     virtual int validFilterMask();
     virtual QueryMaker* limitMaxResultSize( int size );
 
-    //Methods "borrowed" from MemoryQueryMaker
-    void handleResult();
-    void handleResult( const Meta::TrackList &tracks );
-    void handleResult( const Meta::ArtistList &artists );
-    void handleResult( const Meta::AlbumList &albums );
-
     void fetchArtists();
     void fetchAlbums();
     void fetchTracks();
 
 protected:
-    AmpacheServiceCollection * m_collection;
-
     struct Private;
     Private * const d;
 
-    QString m_server;
-    QString m_sessionId;
-    QString m_parentAlbumId;
-    QString m_parentArtistId;
-    QString m_parentService;
-    
-    uint m_dateFilter;
-    QString m_artistFilter;
-    QString m_lastArtistFilter;
-
 public slots:
     void artistDownloadComplete( const KUrl &url, QByteArray data, NetworkAccessManagerProxy::Error e );
     void albumDownloadComplete( const KUrl &url, QByteArray data, NetworkAccessManagerProxy::Error e );
@@ -86,8 +76,15 @@ private:
     AmpacheServiceQueryMaker( const AmpacheServiceQueryMaker& );
     AmpacheServiceQueryMaker& operator= ( const AmpacheServiceQueryMaker& );
 
+    /** Gets the url for the ampache requests.
+        Already adds query items for the dateFilter and the limit.
+    */
+    KUrl getRequestUrl( const QString &action = QString() ) const;
+
+    /*
     template<class PointerType, class ListType>
     void emitProperResult(const ListType& list);
+    */
 };
 
 } //namespace Collections
