@@ -18,24 +18,24 @@
 
 #include "Mp3tunesServiceQueryMaker.h"
 
-#include "Amarok.h"
-#include "Debug.h"
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
 #include "Mp3tunesMeta.h"
 #include "Mp3tunesWorkers.h"
-#include "collection/support/MemoryMatcher.h"
+#include "core-impl/collections/support/MemoryMatcher.h"
 
 #include <threadweaver/Job.h>
 #include <threadweaver/ThreadWeaver.h>
 
 #include <QList>
 
-using namespace Meta;
+using namespace Collections;
 
-struct Mp3tunesServiceQueryMaker::Private {
+class Mp3tunesServiceQueryMaker::Private {
+public:
     enum QueryType { NONE, TRACK, ARTIST, ALBUM, COMPOSER, YEAR, GENRE, CUSTOM };
     QueryType type;
     int maxsize;
-    bool returnDataPtrs;
 };
 
 
@@ -48,20 +48,23 @@ Mp3tunesServiceQueryMaker::Mp3tunesServiceQueryMaker( Mp3tunesServiceCollection
     DEBUG_BLOCK
     m_collection = collection;
     m_sessionId = sessionId;
-    reset();
+
+    d->type = Private::NONE;
+    d->maxsize = -1;
 }
 
 Mp3tunesServiceQueryMaker::Mp3tunesServiceQueryMaker( Mp3tunesLocker * locker, const QString &sessionId, Mp3tunesServiceCollection * collection  )
     : DynamicServiceQueryMaker()
         , m_storedTransferJob( 0 )
         , d( new Private )
-
 {
     DEBUG_BLOCK
     m_collection = collection;
     m_sessionId = sessionId;
     m_locker = locker;
-    reset();
+
+    d->type = Private::NONE;
+    d->maxsize = -1;
 }
 
 Mp3tunesServiceQueryMaker::~Mp3tunesServiceQueryMaker()
@@ -69,26 +72,6 @@ Mp3tunesServiceQueryMaker::~Mp3tunesServiceQueryMaker()
     delete d;
 }
 
-QueryMaker * Mp3tunesServiceQueryMaker::reset()
-{
-    DEBUG_BLOCK
-    d->type = Private::NONE;
-    d->maxsize = -1;
-    d->returnDataPtrs = false;
-    m_parentArtistId.clear();
-    m_parentAlbumId.clear();
-    m_artistFilter.clear();
-
-    return this;
-}
-
-QueryMaker*
-Mp3tunesServiceQueryMaker::setReturnResultAsDataPtrs( bool resultAsDataPtrs )
-{
-    d->returnDataPtrs = resultAsDataPtrs;
-    return this;
-}
-
 void Mp3tunesServiceQueryMaker::run()
 {
     DEBUG_BLOCK
@@ -121,6 +104,7 @@ Mp3tunesServiceQueryMaker::setQueryType( QueryType type )
 {
     switch( type ) {
     case QueryMaker::Artist:
+    case QueryMaker::AlbumArtist:
     {
         DEBUG_BLOCK
         d->type = Private::ARTIST;
@@ -154,11 +138,11 @@ Mp3tunesServiceQueryMaker::setQueryType( QueryType type )
 
 
 
-QueryMaker * Mp3tunesServiceQueryMaker::addMatch( const ArtistPtr & artist )
+QueryMaker * Mp3tunesServiceQueryMaker::addMatch( const Meta::ArtistPtr & artist )
 {
     DEBUG_BLOCK
     if ( m_parentAlbumId.isEmpty() ) {
-        const ServiceArtist * serviceArtist = static_cast< const ServiceArtist * >( artist.data() );
+        const Meta::ServiceArtist * serviceArtist = static_cast< const Meta::ServiceArtist * >( artist.data() );
         m_parentArtistId = QString::number( serviceArtist->id() );
         debug() << "artist parent id set to: " << m_parentArtistId;
     }
@@ -169,7 +153,7 @@ QueryMaker * Mp3tunesServiceQueryMaker::addMatch( const ArtistPtr & artist )
 QueryMaker * Mp3tunesServiceQueryMaker::addMatch(const Meta::AlbumPtr & album)
 {
     DEBUG_BLOCK
-    const ServiceAlbum * serviceAlbum = static_cast< const ServiceAlbum * >( album.data() );
+    const Meta::ServiceAlbum * serviceAlbum = static_cast< const Meta::ServiceAlbum * >( album.data() );
     m_parentAlbumId = QString::number( serviceAlbum->id() );
     debug() << "album parent id set to: " << m_parentAlbumId;
     m_parentArtistId.clear();
@@ -177,55 +161,41 @@ QueryMaker * Mp3tunesServiceQueryMaker::addMatch(const Meta::AlbumPtr & album)
     return this;
 }
 
-template<class PointerType, class ListType>
-void Mp3tunesServiceQueryMaker::emitProperResult( const ListType& list )
-{
-    if ( d->returnDataPtrs ) {
-        DataList data;
-        foreach( PointerType p, list )
-            data << DataPtr::staticCast( p );
-
-        emit newResultReady( m_collection->collectionId(), data );
-    }
-    else
-        emit newResultReady( m_collection->collectionId(), list );
-}
-
 void Mp3tunesServiceQueryMaker::handleResult()
 {
     DEBUG_BLOCK
 }
 
-void Mp3tunesServiceQueryMaker::handleResult( const ArtistList & artists )
+void Mp3tunesServiceQueryMaker::handleResult( const Meta::ArtistList & artists )
 {
     DEBUG_BLOCK
 
     if ( d->maxsize >= 0 && artists.count() > d->maxsize ) {
-        emitProperResult<ArtistPtr, ArtistList>( artists.mid( 0, d->maxsize ) );
+        emit newResultReady( artists.mid( 0, d->maxsize ) );
     } else {
-        emitProperResult<ArtistPtr, ArtistList>( artists );
+        emit newResultReady( artists );
     }
 }
 
-void Mp3tunesServiceQueryMaker::handleResult( const AlbumList &albums )
+void Mp3tunesServiceQueryMaker::handleResult( const Meta::AlbumList &albums )
 {
     DEBUG_BLOCK
 
     if ( d->maxsize >= 0 && albums.count() > d->maxsize ) {
-        emitProperResult<AlbumPtr, AlbumList>( albums.mid( 0, d->maxsize ) );
+        emit newResultReady( albums.mid( 0, d->maxsize ) );
     } else {
-        emitProperResult<AlbumPtr, AlbumList>( albums );
+        emit newResultReady( albums );
     }
 }
 
-void Mp3tunesServiceQueryMaker::handleResult(const TrackList & tracks)
+void Mp3tunesServiceQueryMaker::handleResult(const Meta::TrackList & tracks)
 {
     DEBUG_BLOCK
 
     if ( d->maxsize >= 0 && tracks.count() > d->maxsize ) {
-        emitProperResult<TrackPtr, TrackList>( tracks.mid( 0, d->maxsize ) );
+        emit newResultReady( tracks.mid( 0, d->maxsize ) );
     } else {
-        emitProperResult<TrackPtr, TrackList>( tracks );
+        emit newResultReady( tracks );
     }
 }
 
@@ -237,13 +207,13 @@ void Mp3tunesServiceQueryMaker::fetchArtists()
     {
         debug() << "Artist Filtering";
         Mp3tunesSearchMonkey * searchMonkey = new Mp3tunesSearchMonkey( m_locker, m_artistFilter, Mp3tunesSearchResult::ArtistQuery );
-        connect( searchMonkey, SIGNAL( searchComplete( QList<Mp3tunesLockerArtist> ) ), this, SLOT( artistDownloadComplete( QList<Mp3tunesLockerArtist> ) ) );
+        connect( searchMonkey, SIGNAL(searchComplete(QList<Mp3tunesLockerArtist>)), this, SLOT(artistDownloadComplete(QList<Mp3tunesLockerArtist>)) );
         ThreadWeaver::Weaver::instance()->enqueue( searchMonkey ); //Go!
     } else if( m_locker->sessionValid() )
     {
         debug() << "Artist Fetching";
         Mp3tunesArtistFetcher * artistFetcher = new Mp3tunesArtistFetcher( m_locker );
-        connect( artistFetcher, SIGNAL( artistsFetched( QList<Mp3tunesLockerArtist> ) ), this, SLOT( artistDownloadComplete( QList<Mp3tunesLockerArtist> ) ) );
+        connect( artistFetcher, SIGNAL(artistsFetched(QList<Mp3tunesLockerArtist>)), this, SLOT(artistDownloadComplete(QList<Mp3tunesLockerArtist>)) );
         ThreadWeaver::Weaver::instance()->enqueue( artistFetcher );
     }
 }
@@ -252,7 +222,7 @@ void Mp3tunesServiceQueryMaker::fetchAlbums()
 {
     DEBUG_BLOCK
 
-    AlbumList albums;
+    Meta::AlbumList albums;
 
     debug() << "Fetching Albums for parentArtist id: " << m_parentArtistId;
 
@@ -267,7 +237,7 @@ void Mp3tunesServiceQueryMaker::fetchAlbums()
         handleResult( albums );
     } else if ( m_locker->sessionValid() ) {
         Mp3tunesAlbumWithArtistIdFetcher * albumFetcher = new Mp3tunesAlbumWithArtistIdFetcher( m_locker, m_parentArtistId.toInt() );
-        connect( albumFetcher, SIGNAL( albumsFetched( QList<Mp3tunesLockerAlbum> ) ), this, SLOT( albumDownloadComplete( QList<Mp3tunesLockerAlbum> ) ) );
+        connect( albumFetcher, SIGNAL(albumsFetched(QList<Mp3tunesLockerAlbum>)), this, SLOT(albumDownloadComplete(QList<Mp3tunesLockerAlbum>)) );
 
         ThreadWeaver::Weaver::instance()->enqueue( albumFetcher );
     } else {
@@ -279,8 +249,8 @@ void Mp3tunesServiceQueryMaker::fetchTracks()
 {
     DEBUG_BLOCK
 
-    AlbumList albums;
-    TrackList tracks;
+    Meta::AlbumList albums;
+    Meta::TrackList tracks;
 
     debug() << "album parent id: " << m_parentAlbumId;
     debug() << "artist parent id: " << m_parentArtistId;
@@ -304,12 +274,12 @@ void Mp3tunesServiceQueryMaker::fetchTracks()
         if( !m_parentArtistId.isEmpty() ) {
             debug() << "Creating track w/ artist id Fetch Worker";
             Mp3tunesTrackWithArtistIdFetcher * trackFetcher = new Mp3tunesTrackWithArtistIdFetcher( m_locker, m_parentArtistId.toInt() );
-            connect( trackFetcher, SIGNAL( tracksFetched( QList<Mp3tunesLockerTrack> ) ), this, SLOT( trackDownloadComplete( QList<Mp3tunesLockerTrack> ) ) );
+            connect( trackFetcher, SIGNAL(tracksFetched(QList<Mp3tunesLockerTrack>)), this, SLOT(trackDownloadComplete(QList<Mp3tunesLockerTrack>)) );
             ThreadWeaver::Weaver::instance()->enqueue( trackFetcher ); //Go!
         } else if ( !m_parentAlbumId.isEmpty() ) {
             debug() << "Creating track w/ album id Fetch Worker";
             Mp3tunesTrackWithAlbumIdFetcher * trackFetcher = new Mp3tunesTrackWithAlbumIdFetcher( m_locker, m_parentAlbumId.toInt() );
-            connect( trackFetcher, SIGNAL( tracksFetched( QList<Mp3tunesLockerTrack> ) ), this, SLOT( trackDownloadComplete( QList<Mp3tunesLockerTrack> ) ) );
+            connect( trackFetcher, SIGNAL(tracksFetched(QList<Mp3tunesLockerTrack>)), this, SLOT(trackDownloadComplete(QList<Mp3tunesLockerTrack>)) );
             ThreadWeaver::Weaver::instance()->enqueue( trackFetcher ); //Go!
         }
     } else {
@@ -322,17 +292,17 @@ void Mp3tunesServiceQueryMaker::artistDownloadComplete( QList<Mp3tunesLockerArti
 {
     DEBUG_BLOCK
 
-    ArtistList artists;
+    Meta::ArtistList artists;
 
     debug() << "Received artists";
     foreach(const Mp3tunesLockerArtist &artist, artistList) {
-        ServiceArtist * serviceArtist = new ServiceArtist( artist.artistName() );
+        Meta::ServiceArtist * serviceArtist = new Meta::ServiceArtist( artist.artistName() );
 
         //debug() << "Adding artist: " <<  artist.artistName();
 
         serviceArtist->setId( artist.artistId() );
 
-        ArtistPtr artistPtr( serviceArtist );
+        Meta::ArtistPtr artistPtr( serviceArtist );
 
         artists.push_back( artistPtr );
 
@@ -353,7 +323,7 @@ void Mp3tunesServiceQueryMaker::albumDownloadComplete( QList<Mp3tunesLockerAlbum
 
     debug() << "Received albums";
 
-    AlbumList albums;
+    Meta::AlbumList albums;
     foreach(const Mp3tunesLockerAlbum &album, albumsList) {
 
         QString title = album.albumTitle();
@@ -365,7 +335,7 @@ void Mp3tunesServiceQueryMaker::albumDownloadComplete( QList<Mp3tunesLockerAlbum
 
         bool hasArt = album.hasArt();
 
-        Mp3TunesAlbum * serviceAlbum = new Mp3TunesAlbum( title );
+        Meta::Mp3TunesAlbum * serviceAlbum = new Meta::Mp3TunesAlbum( title );
 
         if ( hasArt )
         {
@@ -379,7 +349,7 @@ void Mp3tunesServiceQueryMaker::albumDownloadComplete( QList<Mp3tunesLockerAlbum
             serviceAlbum->setCoverUrl(coverUrl);
         }
 
-        AlbumPtr albumPtr( serviceAlbum );
+        Meta::AlbumPtr albumPtr( serviceAlbum );
 
         //debug() << "Adding album: " <<  title;
 
@@ -388,7 +358,7 @@ void Mp3tunesServiceQueryMaker::albumDownloadComplete( QList<Mp3tunesLockerAlbum
         m_collection->addAlbum( albumPtr );
         m_collection->releaseLock();
 
-        ArtistPtr artistPtr = m_collection->artistById( album.artistId() );
+        Meta::ArtistPtr artistPtr = m_collection->artistById( album.artistId() );
         if ( artistPtr.data() != 0 )
         {
            //debug() << "Found parent artist";
@@ -409,7 +379,7 @@ void Mp3tunesServiceQueryMaker::trackDownloadComplete( QList<Mp3tunesLockerTrack
     DEBUG_BLOCK
     //debug() << "Received Tracks";
 
-    TrackList tracks;
+    Meta::TrackList tracks;
 
      //so lets figure out what we got here:
 
@@ -419,8 +389,8 @@ void Mp3tunesServiceQueryMaker::trackDownloadComplete( QList<Mp3tunesLockerTrack
         QString title = track.trackTitle();
         if ( title.isEmpty() ) title = "Unknown";
 
-        Mp3TunesTrack * serviceTrack = new Mp3TunesTrack( title );
-        TrackPtr trackPtr( serviceTrack );
+        Meta::Mp3TunesTrack * serviceTrack = new Meta::Mp3TunesTrack( title );
+        Meta::TrackPtr trackPtr( serviceTrack );
 
       //  debug() << "Adding track: " <<  title;
 
@@ -433,7 +403,7 @@ void Mp3tunesServiceQueryMaker::trackDownloadComplete( QList<Mp3tunesLockerTrack
 
         serviceTrack->setTrackNumber( track.trackNumber() );
 
-        serviceTrack->setYear( QString::number( track.albumYear() ) );
+        serviceTrack->setYear( track.albumYear() );
 
         debug() << "setting type: " << Amarok::extension( track.trackFileName() );
         serviceTrack->setType( Amarok::extension( track.trackFileName() ) );
@@ -446,18 +416,18 @@ void Mp3tunesServiceQueryMaker::trackDownloadComplete( QList<Mp3tunesLockerTrack
         QString albumId = QString::number( track.albumId() );
         QString artistId = QString::number( track.artistId() );
 
-        ArtistPtr artistPtr = m_collection->artistById( artistId.toInt() );
+        Meta::ArtistPtr artistPtr = m_collection->artistById( artistId.toInt() );
         if ( artistPtr.data() != 0 ) {
             debug() << "Found parent artist";
-            ServiceArtist *artist = dynamic_cast< ServiceArtist * > ( artistPtr.data() );
+            Meta::ServiceArtist *artist = dynamic_cast< Meta::ServiceArtist * > ( artistPtr.data() );
             serviceTrack->setArtist( artistPtr );
             artist->addTrack( trackPtr );
         }
 
-        AlbumPtr albumPtr = m_collection->albumById( albumId.toInt() );
+        Meta::AlbumPtr albumPtr = m_collection->albumById( albumId.toInt() );
         if ( albumPtr.data() != 0 ) {
             debug() << "Found parent album";
-            ServiceAlbum *album = dynamic_cast< ServiceAlbum * > ( albumPtr.data() );
+            Meta::ServiceAlbum *album = dynamic_cast< Meta::ServiceAlbum * > ( albumPtr.data() );
             serviceTrack->setAlbumPtr( albumPtr );
             album->addTrack( trackPtr );
         }
@@ -478,7 +448,7 @@ QueryMaker * Mp3tunesServiceQueryMaker::addFilter(qint64 value, const QString &
     DEBUG_BLOCK
     //debug() << "value: " << value;
     //for now, only accept artist filters
-    if ( value == valArtist ) {
+    if ( value == Meta::valArtist ) {
         //debug() << "Filter: " << filter;
         m_artistFilter = filter;
     }
