@@ -19,8 +19,6 @@
 
 #include "MainToolbar.h"
 
-#include "amarokconfig.h"
-
 #include "App.h"
 #include "ActionClasses.h"
 #include "core/support/Amarok.h"
@@ -29,20 +27,18 @@
 #include "GlobalCurrentTrackActions.h"
 #include "MainWindow.h"
 #include "SvgHandler.h"
-
+#include "amarokconfig.h"
 #include "amarokurls/AmarokUrl.h"
 #include "amarokurls/AmarokUrlHandler.h"
-
 #include "browsers/collectionbrowser/CollectionWidget.h"
-
 #include "core/capabilities/ActionsCapability.h"
 #include "core/capabilities/BookmarkThisCapability.h"
+#include "core/meta/Meta.h"
 #include "core/meta/support/MetaUtility.h"
 #include "core-impl/capabilities/timecode/TimecodeLoadCapability.h"
-
 #include "playlist/PlaylistActions.h"
 #include "playlist/PlaylistModelStack.h"
-
+#include "playlist/PlaylistController.h"
 #include "widgets/AnimatedLabelStack.h"
 #include "widgets/PlayPauseButton.h"
 #include "widgets/SliderWidget.h"
@@ -80,6 +76,7 @@ static const int track_action_spacing = 6;
 MainToolbar::MainToolbar( QWidget *parent )
     : QToolBar( i18n( "Main Toolbar" ), parent )
     , m_lastTime( -1 )
+    , m_trackBarAnimationTimer( 0 )
 {
     DEBUG_BLOCK
     setObjectName( "MainToolbar" );
@@ -107,7 +104,7 @@ MainToolbar::MainToolbar( QWidget *parent )
     QVBoxLayout *vl = new QVBoxLayout( info );
 
     QFont fnt = QApplication::font(); // don't use the toolbar font. Often small to support icons only.
-    if ( fnt.pointSize() > 0 )
+    if( fnt.pointSize() > 0 )
         fnt.setPointSize( qRound(fnt.pointSize() * track_fontsize_factor) );
     const int fntH = QFontMetrics( QApplication::font() ).height();
 
@@ -117,7 +114,7 @@ MainToolbar::MainToolbar( QWidget *parent )
     m_prev.key = 0;
     m_prev.label = new AnimatedLabelStack(QStringList(), info);
     m_prev.label->setFont( fnt );
-    if ( layoutDirection() == Qt::LeftToRight )
+    if( layoutDirection() == Qt::LeftToRight )
         m_prev.label->setPadding( m_skip_right.width() + skipPadding + skipMargin, 0 );
     else
         m_prev.label->setPadding( 0, m_skip_right.width() + skipPadding + skipMargin );
@@ -140,7 +137,7 @@ MainToolbar::MainToolbar( QWidget *parent )
     m_next.key = 0;
     m_next.label = new AnimatedLabelStack(QStringList(), info);
     m_next.label->setFont( fnt );
-    if ( layoutDirection() == Qt::LeftToRight )
+    if( layoutDirection() == Qt::LeftToRight )
         m_next.label->setPadding( 0, m_skip_left.width() + skipPadding + skipMargin );
     else
         m_next.label->setPadding( m_skip_left.width() + skipPadding + skipMargin, 0 );
@@ -166,8 +163,8 @@ MainToolbar::MainToolbar( QWidget *parent )
     m_timeLabel->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
 
     m_slider = new Amarok::TimeSlider( info );
-    connect( m_slider, SIGNAL(sliderReleased(int)), The::engineController(), SLOT(seek(int)) );
-    connect( m_slider, SIGNAL(valueChanged(int)), SLOT( setLabelTime(int) ) );
+    connect( m_slider, SIGNAL(sliderReleased(int)), The::engineController(), SLOT(seekTo(int)) );
+    connect( m_slider, SIGNAL(valueChanged(int)), SLOT(setLabelTime(int)) );
     connect( App::instance(), SIGNAL(settingsChanged()), SLOT(layoutProgressBar()) );
 
     m_remainingTimeLabel = new QLabel( info );
@@ -200,7 +197,7 @@ MainToolbar::MainToolbar( QWidget *parent )
 void
 MainToolbar::addBookmark( const QString &name, int milliSeconds )
 {
-    if ( m_slider )
+    if( m_slider )
         m_slider->drawTriangle( name, milliSeconds, false );
 }
 
@@ -209,7 +206,7 @@ static void adjustLabelPos( QWidget *label, int targetX )
 {
     QRect r = label->geometry();
     int d = targetX - r.x();
-    if ( d )
+    if( d )
     {
         r.translate( qMin( qAbs(d), r.width()/6 ) * (d > 0 ? 1 : -1), 0 );
         label->setGeometry( r );
@@ -224,25 +221,25 @@ MainToolbar::animateTrackLabels()
     int off = -m_current.label->parentWidget()->geometry().x();
     adjustLabelPos( m_prev.label, m_prev.rect.x() + off );
     m_prev.label->setOpacity( prevOpacity );
-    if (done)
+    if(done)
         done = m_prev.label->geometry().x() == m_prev.rect.x() + off;
 
     adjustLabelPos( m_current.label, m_current.rect.x() + off );
-    if (done)
+    if(done)
         done = m_current.label->geometry().x() == m_current.rect.x() + off;
 
     adjustLabelPos( m_next.label, m_next.rect.x() + off );
     m_next.label->setOpacity( nextOpacity );
-    if (done)
+    if(done)
         done = m_next.label->geometry().x() == m_next.rect.x() + off;
 
     adjustLabelPos( m_dummy.label, m_dummy.targetX );
-    if ( m_dummy.label->geometry().x() == m_dummy.targetX )
+    if( m_dummy.label->geometry().x() == m_dummy.targetX )
         m_dummy.label->hide();
     else
         done = false;
 
-    if ( done )
+    if( done )
     {
         killTimer( m_trackBarAnimationTimer );
         setCurrentTrackActionsVisible( true );
@@ -286,7 +283,7 @@ MainToolbar::muteStateChanged( bool mute )
 void
 MainToolbar::filter( const QString &string )
 {
-    if ( CollectionWidget::instance() )
+    if( CollectionWidget::instance() )
         CollectionWidget::instance()->setFilter( string );
 }
 
@@ -298,7 +295,7 @@ MainToolbar::layoutProgressBar()
 
     const int bw = AmarokConfig::showMoodbarInSlider() ? 10 : 6;
     int w = bw;
-    if ( size().width() < limit )
+    if( size().width() < limit )
     {
         w = (limit<<7)/size().width();
         w = w*w*w*bw;
@@ -307,10 +304,10 @@ MainToolbar::layoutProgressBar()
 
     w = r.width() / w;
     int tlW = m_timeLabel->width();
-    if ( tlW + timeLabelMargin > w )
+    if( tlW + timeLabelMargin > w )
         w = tlW;
     int rtlW = m_remainingTimeLabel->width();
-    if ( rtlW + timeLabelMargin > w )
+    if( rtlW + timeLabelMargin > w )
         w = rtlW;
 
     QRect pb = r.adjusted( w, 0, -w, 0 );
@@ -319,7 +316,7 @@ MainToolbar::layoutProgressBar()
     QRect tlR( 0, 0, tlW, r.height() );
     QRect rtlR( 0, 0, rtlW, r.height() );
 
-    if ( layoutDirection() == Qt::LeftToRight )
+    if( layoutDirection() == Qt::LeftToRight )
     {
         tlR.moveTopRight( pb.topLeft() - QPoint( timeLabelMargin, 0 ) );
         rtlR.moveTopLeft( pb.topRight() + QPoint( timeLabelMargin, 0 )  );
@@ -344,7 +341,7 @@ MainToolbar::layoutTrackBar()
     r.setWidth( r.width() / 3);
     int d = r.width();
 
-    if ( layoutDirection() == Qt::RightToLeft )
+    if( layoutDirection() == Qt::RightToLeft )
     {
         d = -d;
         r.moveRight( m_trackBarSpacer->geometry().right() );
@@ -387,11 +384,11 @@ MainToolbar::updateCurrentTrackActions()
     if( track )
     {
         QScopedPointer< Capabilities::ActionsCapability > ac( track->create<Capabilities::ActionsCapability>() );
-        if ( ac )
+        if( ac )
             actions << ac->actions();
 
         QScopedPointer< Capabilities::BookmarkThisCapability > btc( track->create<Capabilities::BookmarkThisCapability>() );
-        if ( btc && btc->bookmarkAction() )
+        if( btc && btc->bookmarkAction() )
             actions << btc->bookmarkAction();
     }
 
@@ -403,7 +400,7 @@ MainToolbar::updateCurrentTrackActions()
     const int n = actions.count() / 2;
     for ( int i = 0; i < actions.count(); ++i )
     {
-        if ( i == n )
+        if( i == n )
             hbl->addStretch( 10 );
         btn = new TrackActionButton( m_current.label, actions.at(i) );
         if( !actions.at(i)->parent() ) // see documentation of ActionsCapability::actions
@@ -420,29 +417,29 @@ MainToolbar::updateCurrentTrackActions()
 static QStringList metadata( Meta::TrackPtr track )
 {
     QStringList list;
-    if ( track )
+    if( track )
     {
         bool noTags = false;
         QString title = track->prettyName();
-        if (( noTags = title.isEmpty() )) // should not happen
+        if(( noTags = title.isEmpty() )) // should not happen
             title = track->prettyUrl();
-        if (( noTags = title.isEmpty() )) // should never happen
+        if(( noTags = title.isEmpty() )) // should never happen
             title = track->playableUrl().url();
-        if (( noTags = title.isEmpty() )) // sth's MEGA wrong ;-)
+        if(( noTags = title.isEmpty() )) // sth's MEGA wrong ;-)
             title = "???";
 
         // no tags -> probably filename. try to strip the suffix
-        if ( noTags || track->name().isEmpty() )
+        if( noTags || track->name().isEmpty() )
         {
             noTags = true;
             int dot = title.lastIndexOf('.');
-            if ( dot > 0 && title.length() - dot < 6 )
+            if( dot > 0 && title.length() - dot < 6 )
                 title = title.left( dot );
         }
 
         // the track has no tags, is long or contains tags other than the titlebar
         // ==> separate
-        if ( noTags || title.length() > 50 ||
+        if( noTags || title.length() > 50 ||
              (HAS_TAG(artist) && title.CONTAINS_TAG(artist)) ||
              (HAS_TAG(composer) && title.CONTAINS_TAG(composer)) ||
              (HAS_TAG(album) && title.CONTAINS_TAG(album)) )
@@ -456,7 +453,7 @@ static QStringList metadata( Meta::TrackPtr track )
             {
                 // check whether this entry is only a number, i.e. probably year or track #
                 i->toInt( &ok );
-                if ( ok )
+                if( ok )
                     i = list.erase( i );
                 else
                     ++i;
@@ -467,11 +464,11 @@ static QStringList metadata( Meta::TrackPtr track )
             list << title;
         }
 
-        if ( HAS_TAG(artist) && !list.CONTAINS_TAG(artist) )
+        if( HAS_TAG(artist) && !list.CONTAINS_TAG(artist) )
             list << TAG(artist);
-        else if ( HAS_TAG(composer) && !list.CONTAINS_TAG(composer) )
+        else if( HAS_TAG(composer) && !list.CONTAINS_TAG(composer) )
             list << TAG(composer);
-        if ( HAS_TAG(album) && !list.CONTAINS_TAG(album) )
+        if( HAS_TAG(album) && !list.CONTAINS_TAG(album) )
             list << TAG(album);
 
         /* other tags
@@ -501,7 +498,7 @@ static QStringList metadata( Meta::TrackPtr track )
 void
 MainToolbar::updatePrevAndNext()
 {
-    if ( !The::engineController()->currentTrack() )
+    if( !The::engineController()->currentTrack() )
     {
         m_prev.key = 0L;
         m_prev.label->setForegroundRole( foregroundRole() );
@@ -532,7 +529,7 @@ MainToolbar::updatePrevAndNext()
     m_next.label->setOpacity( nextOpacity );
     m_next.label->setData( metadata( track ) );
     m_next.label->setCursor( track ? Qt::PointingHandCursor : Qt::ArrowCursor );
-    if ( hadKey != bool(m_next.key) )
+    if( hadKey != bool(m_next.key) )
         needUpdate = true;
 
     hadKey = bool(m_prev.key);
@@ -542,18 +539,18 @@ MainToolbar::updatePrevAndNext()
     m_next.label->setOpacity( prevOpacity );
     m_prev.label->setData( metadata( track ) );
     m_prev.label->setCursor( track ? Qt::PointingHandCursor : Qt::ArrowCursor );
-    if ( hadKey != bool(m_prev.key) )
+    if( hadKey != bool(m_prev.key) )
         needUpdate = true;
 
-    // we may have disbaled it as otherwise the current label gets updated one eventcycle before prev & next
+    // we may have disabled it as otherwise the current label gets updated one eventcycle before prev & next
     // see ::engineTrackChanged()
     m_current.label->setUpdatesEnabled( true );
 
-    if ( needUpdate )
+    if( needUpdate )
         update();
 
     // unanimated change, probably by sliding the bar - fix label positions
-    if ( !m_trackBarAnimationTimer )
+    if( !m_trackBarAnimationTimer )
         layoutTrackBar();
 }
 
@@ -562,16 +559,16 @@ MainToolbar::updateBookmarks( const QString *BookmarkName )
 {
     // DEBUG_BLOCK
     m_slider->clearTriangles();
-    if ( Meta::TrackPtr track = The::engineController()->currentTrack() )
+    if( Meta::TrackPtr track = The::engineController()->currentTrack() )
     {
-        if ( track->hasCapabilityInterface( Capabilities::Capability::LoadTimecode ) )
+        if( track->has<Capabilities::TimecodeLoadCapability>() )
         {
             Capabilities::TimecodeLoadCapability *tcl = track->create<Capabilities::TimecodeLoadCapability>();
             BookmarkList list = tcl->loadTimecodes();
             // debug() << "found " << list.count() << " timecodes on this track";
             foreach( AmarokUrlPtr url, list )
             {
-                if ( url->command() == "play" && url->args().keys().contains( "pos" ) )
+                if( url->command() == "play" && url->args().keys().contains( "pos" ) )
                 {
                     int pos = url->args().value( "pos" ).toDouble() * 1000;
                     debug() << "showing timecode: " << url->name() << " at " << pos ;
@@ -586,15 +583,15 @@ MainToolbar::updateBookmarks( const QString *BookmarkName )
 void
 MainToolbar::trackChanged( Meta::TrackPtr track )
 {
-    if ( !isVisible() || (m_trackBarAnimationTimer && track && track.data() == m_current.key) )
+    if( !isVisible() || (m_trackBarAnimationTimer && track && track.data() == m_current.key) )
         return;
-    if ( m_trackBarAnimationTimer )
+    if( m_trackBarAnimationTimer )
     {
         killTimer( m_trackBarAnimationTimer );
         m_trackBarAnimationTimer = 0;
     }
 
-    if ( track )
+    if( track )
     {
         m_current.key = track.data();
         m_current.uidUrl = track->uidUrl();
@@ -609,21 +606,21 @@ MainToolbar::trackChanged( Meta::TrackPtr track )
         r.setWidth( r.width() / 3 );
         int d = r.width();
 
-        if ( layoutDirection() == Qt::RightToLeft )
+        if( layoutDirection() == Qt::RightToLeft )
         {
             d = -d;
             r.moveRight( m_trackBarSpacer->geometry().right() );
         }
 
-        if ( isVisible() &&  m_current.label->geometry().x() == r.x() + d )
+        if( isVisible() &&  m_current.label->geometry().x() == r.x() + d )
         {
-            if ( m_current.key == m_next.key && m_current.key != m_prev.key )
+            if( m_current.key == m_next.key && m_current.key != m_prev.key )
             {
                 setCurrentTrackActionsVisible( false );
 
                 // left
                 m_dummy.targetX = r.x() - d;
-//                 if ( d < 0 ) // rtl
+//                 if( d < 0 ) // rtl
 //                     m_dummy.targetX -= d;
                 m_dummy.label->setGeometry( r );
                 m_dummy.label->setData( m_prev.label->data() );
@@ -641,7 +638,7 @@ MainToolbar::trackChanged( Meta::TrackPtr track )
                 animateTrackLabels();
                 m_trackBarAnimationTimer = startTimer( 40 );
             }
-            else if ( m_current.key == m_prev.key )
+            else if( m_current.key == m_prev.key )
             {
                 setCurrentTrackActionsVisible( false );
 
@@ -685,7 +682,7 @@ MainToolbar::trackChanged( Meta::TrackPtr track )
     const int pbsH = qMax( m_timeLabel->sizeHint().height(), m_slider->sizeHint().height() );
     m_progressBarSpacer->changeSize(0, pbsH, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
 
-    QTimer::singleShot( 0, this, SLOT( updatePrevAndNext() ) );
+    QTimer::singleShot( 0, this, SLOT(updatePrevAndNext()) );
 }
 
 void
@@ -696,7 +693,7 @@ MainToolbar::trackLengthChanged( qint64 ms )
 
     // get the urlid of the current track as the engine might stop and start several times
     // when skipping last.fm tracks, so we need to know if we are still on the same track...
-    if ( Meta::TrackPtr track = The::engineController()->currentTrack() )
+    if( Meta::TrackPtr track = The::engineController()->currentTrack() )
         m_current.uidUrl = track->uidUrl();
 
     updateBookmarks( 0 );
@@ -716,36 +713,44 @@ MainToolbar::showEvent( QShowEvent *ev )
 {
     EngineController *engine = The::engineController();
 
-    connect( engine, SIGNAL( stopped( qint64, qint64 ) ),
-             this, SLOT( stopped() ) );
-    connect( engine, SIGNAL( paused() ),
-             this, SLOT( paused() ) );
-    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
-             this, SLOT( playing() ) );
+    connect( engine, SIGNAL(stopped(qint64,qint64)),
+             this, SLOT(stopped()) );
+    connect( engine, SIGNAL(paused()),
+             this, SLOT(paused()) );
+    connect( engine, SIGNAL(trackPlaying(Meta::TrackPtr)),
+             this, SLOT(playing()) );
 
-    connect( engine, SIGNAL( trackChanged( Meta::TrackPtr ) ),
-             this, SLOT( trackChanged( Meta::TrackPtr ) ) );
-    connect( engine, SIGNAL( trackMetadataChanged( Meta::TrackPtr ) ),
-             this, SLOT( trackChanged( Meta::TrackPtr ) ) );
-    connect( engine, SIGNAL( trackLengthChanged( qint64 ) ),
-             this, SLOT( trackLengthChanged( qint64 ) ) );
-    connect( engine, SIGNAL( trackPositionChanged( qint64, bool ) ),
-             this, SLOT( trackPositionChanged( qint64, bool ) ) );
-    connect( engine, SIGNAL( volumeChanged( int ) ),
-             this, SLOT( volumeChanged( int ) ) );
-    connect( engine, SIGNAL( muteStateChanged( bool ) ),
-             this, SLOT( muteStateChanged( bool ) ) );
+    connect( engine, SIGNAL(trackChanged(Meta::TrackPtr)),
+             this, SLOT(trackChanged(Meta::TrackPtr)) );
+    connect( engine, SIGNAL(trackMetadataChanged(Meta::TrackPtr)),
+             this, SLOT(trackChanged(Meta::TrackPtr)) );
+    connect( engine, SIGNAL(trackLengthChanged(qint64)),
+             this, SLOT(trackLengthChanged(qint64)) );
+    connect( engine, SIGNAL(trackPositionChanged(qint64,bool)),
+             this, SLOT(trackPositionChanged(qint64,bool)) );
+    connect( engine, SIGNAL(volumeChanged(int)),
+             this, SLOT(volumeChanged(int)) );
+    connect( engine, SIGNAL(muteStateChanged(bool)),
+             this, SLOT(muteStateChanged(bool)) );
 
-    connect ( The::playlistController(), SIGNAL( changed()),
-              this, SLOT( updatePrevAndNext() ) );
+    // We need the three changed signals:
+    // 1. the playlist changes (PlaylistController)
+    // 2. the sorting of the playlist changed (Playlist)
+    // 3. The navigator changed to e.g. dynamic mode (PlaylistActions)
 
-    connect ( The::playlist()->qaim(), SIGNAL( queueChanged() ),
-              this, SLOT( updatePrevAndNext() ) );
+    connect( The::playlistController(), SIGNAL(changed()),
+              this, SLOT(updatePrevAndNext()) );
 
-    connect ( The::amarokUrlHandler(), SIGNAL( timecodesUpdated(const QString*) ),
-              this, SLOT( updateBookmarks(const QString*) ) );
-    connect ( The::amarokUrlHandler(), SIGNAL( timecodeAdded(const QString&, int) ),
-              this, SLOT( addBookmark(const QString&, int) ) );
+    connect( The::playlist()->qaim(), SIGNAL(queueChanged()),
+              this, SLOT(updatePrevAndNext()) );
+
+    connect( The::playlistActions(), SIGNAL(navigatorChanged()),
+                 this, SLOT(updatePrevAndNext()) );
+
+    connect( The::amarokUrlHandler(), SIGNAL(timecodesUpdated(const QString*)),
+              this, SLOT(updateBookmarks(const QString*)) );
+    connect( The::amarokUrlHandler(), SIGNAL(timecodeAdded(QString,int)),
+              this, SLOT(addBookmark(QString,int)) );
 
     QToolBar::showEvent( ev );
     trackChanged( The::engineController()->currentTrack() );
@@ -762,19 +767,19 @@ MainToolbar::hideEvent( QHideEvent *ev )
 
     disconnect( The::engineController(), 0, this, 0 );
 
-    disconnect ( The::playlistController(), SIGNAL( changed()),
-                 this, SLOT( updatePrevAndNext() ) );
+    disconnect( The::playlistController(), SIGNAL(changed()),
+                 this, SLOT(updatePrevAndNext()) );
 
-    disconnect ( Playlist::ModelStack::instance()->bottom(), SIGNAL( queueChanged() ),
-                 this, SLOT( updatePrevAndNext() ) );
+    disconnect( The::playlist()->qaim(), SIGNAL(queueChanged()),
+                 this, SLOT(updatePrevAndNext()) );
 
-    disconnect ( The::playlistActions(), SIGNAL( navigatorChanged()),
-                 this, SLOT( updatePrevAndNext() ) );
+    disconnect( The::playlistActions(), SIGNAL(navigatorChanged()),
+                 this, SLOT(updatePrevAndNext()) );
 
-    disconnect ( The::amarokUrlHandler(), SIGNAL( timecodesUpdated(const QString*) ),
-                 this, SLOT( updateBookmarks(const QString*) ) );
-    disconnect ( The::amarokUrlHandler(), SIGNAL( timecodeAdded(const QString&, int) ),
-                 this, SLOT( addBookmark(const QString&, int) ) );
+    disconnect( The::amarokUrlHandler(), SIGNAL(timecodesUpdated(const QString*)),
+                 this, SLOT(updateBookmarks(const QString*)) );
+    disconnect( The::amarokUrlHandler(), SIGNAL(timecodeAdded(QString,int)),
+                 this, SLOT(addBookmark(QString,int)) );
 }
 
 void
@@ -788,17 +793,17 @@ MainToolbar::paintEvent( QPaintEvent *ev )
     Skip *left = &m_prev;
     Skip *right = &m_next;
 
-    if ( layoutDirection() == Qt::RightToLeft )
+    if( layoutDirection() == Qt::RightToLeft )
     {
         left = &m_next;
         right = &m_prev;
     }
 
-    if ( left->key )
+    if( left->key )
         p.drawPixmap( left->rect.left() + skipMargin,
                       left->rect.y() + ( left->rect.height() - m_skip_left.height() ) /2,
                       m_skip_left );
-    if ( right->key )
+    if( right->key )
         p.drawPixmap( right->rect.right() - ( m_skip_right.width() + skipMargin ),
                       right->rect.y() + ( right->rect.height() - m_skip_right.height() ) /2,
                       m_skip_right );
@@ -809,7 +814,7 @@ MainToolbar::paintEvent( QPaintEvent *ev )
 void
 MainToolbar::resizeEvent( QResizeEvent *ev )
 {
-    if ( ev->size().width() > 0 && ev->size().width() != ev->oldSize().width() )
+    if( ev->size().width() > 0 && ev->size().width() != ev->oldSize().width() )
     {
         layoutProgressBar();
         layoutTrackBar();
@@ -819,14 +824,14 @@ MainToolbar::resizeEvent( QResizeEvent *ev )
 void
 MainToolbar::setCurrentTrackActionsVisible( bool vis )
 {
-    if ( m_current.actionsVisible == vis )
+    if( m_current.actionsVisible == vis )
         return;
     m_current.actionsVisible = vis;
     QLayoutItem *item;
     for ( int i = 0; i < m_current.label->layout()->count(); ++i )
     {
         item = m_current.label->layout()->itemAt( i );
-        if ( item->widget() )
+        if( item->widget() )
             item->widget()->setVisible( vis );
     }
 }
@@ -836,11 +841,11 @@ const char * timeString[4] = { "3:33", "33:33", "3:33:33", "33:33:33" };
 static inline int
 timeFrame( int secs )
 {
-    if ( secs < 10*60 ) // 9:59
+    if( secs < 10*60 ) // 9:59
         return 0;
-    if ( secs < 60*60 ) // 59:59
+    if( secs < 60*60 ) // 59:59
         return 1;
-    if ( secs < 10*60*60 ) // 9:59:59
+    if( secs < 10*60*60 ) // 9:59:59
         return 2;
     return 3; // 99:59:59
 }
@@ -849,7 +854,7 @@ void
 MainToolbar::setLabelTime( int ms )
 {
     bool relayout = false;
-    if ( ms < 0 ) // clear
+    if( ms < 0 ) // clear
     {
         m_timeLabel->hide();
         m_remainingTimeLabel->hide();
@@ -857,20 +862,20 @@ MainToolbar::setLabelTime( int ms )
         m_lastRemainingTime = -1;
         relayout = true;
     }
-    else if ( isVisible() ) // no need to do expensive stuff - it's updated every second anyway
+    else if( isVisible() ) // no need to do expensive stuff - it's updated every second anyway
     {
 
         const int secs = ms/1000;
         const int remainingSecs =  m_slider->maximum() > 0 ? (m_slider->maximum() - ms) / 1000 : 0;
 
-        if ( secs == m_lastTime && remainingSecs == m_lastRemainingTime )
+        if( secs == m_lastTime && remainingSecs == m_lastRemainingTime )
             return;
 
         m_timeLabel->setText( Meta::secToPrettyTime( secs ) );
 
         // -- determine fix the timeLabel width at a sensible maximum
         int tf = timeFrame( secs );
-        if ( m_lastTime < 0 || tf != timeFrame( m_lastTime ) )
+        if( m_lastTime < 0 || tf != timeFrame( m_lastTime ) )
         {
             const int w = QFontMetrics( m_timeLabel->font() ).width( timeString[tf] );
             m_timeLabel->setFixedWidth( w );
@@ -878,11 +883,11 @@ MainToolbar::setLabelTime( int ms )
         }
         m_timeLabel->show();
 
-        if ( remainingSecs > 0 )
+        if( remainingSecs > 0 )
         {
             m_remainingTimeLabel->setText( '-' + Meta::secToPrettyTime( remainingSecs ) );
             tf = timeFrame( remainingSecs );
-            if ( m_lastRemainingTime < 0 || tf != timeFrame( m_lastRemainingTime ) )
+            if( m_lastRemainingTime < 0 || tf != timeFrame( m_lastRemainingTime ) )
             {
                 const int w = QFontMetrics( m_remainingTimeLabel->font() ).width( QString("-") + timeString[tf] );
                 m_remainingTimeLabel->setFixedWidth( w );
@@ -898,14 +903,14 @@ MainToolbar::setLabelTime( int ms )
 
     }
 
-    if (relayout)
+    if(relayout)
         layoutProgressBar();
 }
 
 void
 MainToolbar::timerEvent( QTimerEvent *ev )
 {
-    if ( ev->timerId() == m_trackBarAnimationTimer )
+    if( ev->timerId() == m_trackBarAnimationTimer )
         animateTrackLabels();
     else
         QToolBar::timerEvent( ev );
@@ -914,20 +919,20 @@ MainToolbar::timerEvent( QTimerEvent *ev )
 bool
 MainToolbar::eventFilter( QObject *o, QEvent *ev )
 {
-    if ( ev->type() == QEvent::MouseMove )
+    if( ev->type() == QEvent::MouseMove )
     {
         QMouseEvent *mev = static_cast<QMouseEvent*>(ev);
-        if ( mev->buttons() & Qt::LeftButton )
-        if ( o == m_current.label || o == m_prev.label || o == m_next.label )
+        if( mev->buttons() & Qt::LeftButton )
+        if( o == m_current.label || o == m_prev.label || o == m_next.label )
         {
             setCurrentTrackActionsVisible( false );
             const int x = mev->globalPos().x();
             int d = x - m_drag.lastX;
             m_drag.lastX = x;
             const int globalDist = qAbs( x - m_drag.startX );
-            if ( globalDist > m_drag.max )
+            if( globalDist > m_drag.max )
                 m_drag.max = globalDist;
-            if ( globalDist > m_prev.label->width() )
+            if( globalDist > m_prev.label->width() )
                 return false; // constrain to one item width
 
             m_current.label->setGeometry( m_current.label->geometry().translated( d, 0 ) );
@@ -936,11 +941,11 @@ MainToolbar::eventFilter( QObject *o, QEvent *ev )
         }
         return false;
     }
-    if ( ev->type() == QEvent::MouseButtonPress )
+    if( ev->type() == QEvent::MouseButtonPress )
     {
         QMouseEvent *mev = static_cast<QMouseEvent*>(ev);
-        if ( mev->button() == Qt::LeftButton )
-        if ( o == m_current.label || o == m_prev.label || o == m_next.label )
+        if( mev->button() == Qt::LeftButton )
+        if( o == m_current.label || o == m_prev.label || o == m_next.label )
         {
             static_cast<QWidget*>(o)->setCursor( Qt::SizeHorCursor );
             m_drag.max = 0;
@@ -948,11 +953,11 @@ MainToolbar::eventFilter( QObject *o, QEvent *ev )
         }
         return false;
     }
-    if ( ev->type() == QEvent::MouseButtonRelease )
+    if( ev->type() == QEvent::MouseButtonRelease )
     {
         QMouseEvent *mev = static_cast<QMouseEvent*>(ev);
-        if ( mev->button() == Qt::LeftButton )
-        if ( o == m_current.label || o == m_prev.label || o == m_next.label )
+        if( mev->button() == Qt::LeftButton )
+        if( o == m_current.label || o == m_prev.label || o == m_next.label )
         {
             const int x = mev->globalPos().x();
             const int d = m_drag.startX - x;
@@ -965,7 +970,7 @@ MainToolbar::eventFilter( QObject *o, QEvent *ev )
 
             // if this was a _real_ drag, silently release the mouse
             const bool silentRelease = m_drag.max > 25;
-            if ( silentRelease )
+            if( silentRelease )
             {   // this is a drag, release secretly
                 o->blockSignals( true );
                 o->removeEventFilter( this );
@@ -979,15 +984,15 @@ MainToolbar::eventFilter( QObject *o, QEvent *ev )
             // if moved "far enough" jump to prev/next track
             // NOTICE the labels shall snap back _after_ the track has changed (in case)
             // as this is not reliable, we force a timered snapback sa well
-            if ( d > limit )
+            if( d > limit )
             {
                 The::playlistActions()->next();
-                QTimer::singleShot(500, this, SLOT( layoutTrackBar() ) );
+                QTimer::singleShot(500, this, SLOT(layoutTrackBar()) );
             }
-            else if ( d < -limit )
+            else if( d < -limit )
             {
                 The::playlistActions()->back();
-                QTimer::singleShot(500, this, SLOT( layoutTrackBar() ) );
+                QTimer::singleShot(500, this, SLOT(layoutTrackBar()) );
             }
             else
                 layoutTrackBar();
@@ -997,26 +1002,26 @@ MainToolbar::eventFilter( QObject *o, QEvent *ev )
         return false;
     }
 
-    if ( ev->type() == QEvent::Enter )
+    if( ev->type() == QEvent::Enter )
     {
-        if (o == m_next.label && m_next.key)
+        if(o == m_next.label && m_next.key)
             m_next.label->setOpacity( 255 );
-        else if (o == m_prev.label && m_prev.key)
+        else if(o == m_prev.label && m_prev.key)
             m_prev.label->setOpacity( 255 );
-        else if ( o->parent() == m_current.label ) // trackaction
+        else if( o->parent() == m_current.label ) // trackaction
         {
             QEvent e( QEvent::Leave );
             QCoreApplication::sendEvent( m_current.label, &e );
         }
         return false;
     }
-    if ( ev->type() == QEvent::Leave )
+    if( ev->type() == QEvent::Leave )
     {
-        if (o == m_next.label && m_next.key)
+        if(o == m_next.label && m_next.key)
             m_next.label->setOpacity( nextOpacity );
-        else if (o == m_prev.label && m_prev.key)
+        else if(o == m_prev.label && m_prev.key)
             m_prev.label->setOpacity( prevOpacity );
-        else if ( o->parent() == m_current.label &&  // trackaction
+        else if( o->parent() == m_current.label &&  // trackaction
                   m_current.label->rect().contains( m_current.label->mapFromGlobal(QCursor::pos()) ) )
         {
             QEvent e( QEvent::Enter );
