@@ -17,8 +17,9 @@
 #ifndef TOKENWITHLAYOUT_H
 #define TOKENWITHLAYOUT_H
 
+#include "widgets/Token.h"
+
 #include <QWeakPointer>
-#include <Token.h>
 
 class LayoutEditDialog;
 
@@ -40,7 +41,7 @@ signals:
 class TokenWithLayoutFactory : public TokenFactory
 {
 public:
-    virtual Token * createToken( const QString &text, const QString &iconName, qint64 value, QWidget *parent = 0 );
+    virtual Token * createToken( const QString &text, const QString &iconName, qint64 value, QWidget *parent = 0 ) const;
 };
 
 /**
