@@ -60,8 +60,9 @@ public:
         coll->mc->releaseLock();
         return true;
     }
-    void copyUrlsToCollection(const QMap<Meta::TrackPtr, KUrl> &sources)
+    void copyUrlsToCollection(const QMap<Meta::TrackPtr, KUrl> &sources, const Transcoding::Configuration& conf)
     {
+        Q_UNUSED( conf )
         trackCopyCount << sources.count();
         foreach( const Meta::TrackPtr &track, sources.keys() )
         {
@@ -75,10 +76,10 @@ class MyCollectionTestImpl : public CollectionTestImpl
 public:
     MyCollectionTestImpl( const QString &id ) : CollectionTestImpl( id ) {}
 
-    CollectionLocation* location() const
+    CollectionLocation* location()
     {
         MyCollectionLocation *r = new MyCollectionLocation();
-        r->coll = const_cast<MyCollectionTestImpl*>( this );
+        r->coll = this;
         return r;
     }
 };
@@ -92,11 +93,10 @@ void addMockTrack( Collections::CollectionTestImpl *coll, const QString &trackNa
     Meta::TrackPtr trackPtr( track );
     EXPECT_CALL( *track, name() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName ) );
     EXPECT_CALL( *track, uidUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName + "_" + artistName + "_" + albumName ) );
-    EXPECT_CALL( *track, isPlayable() ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
     EXPECT_CALL( *track, playableUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( KUrl( '/' + track->uidUrl() ) ) );
     coll->mc->addTrack( trackPtr );
 
-    Meta::AlbumPtr albumPtr = coll->mc->albumMap().value( albumName );
+    Meta::AlbumPtr albumPtr = coll->mc->albumMap().value( albumName, QString() /* no album artist */ );
     Meta::MockAlbum *album;
     Meta::TrackList albumTracks;
     if( albumPtr )
